//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "dialog_info.hpp"
#include "config.hpp"

#include <QVBoxLayout>
#include <QLocale>
#include <QFileInfo>
#include <QFile>
#include <QDir>

#include <iostream>


Dialog_Info::Dialog_Info (
	QWidget * parent ) :
QDialog ( parent, Qt::Tool ),
_title ( this ),
_tabs ( this )
{
	setWindowTitle ( tr ( "Info" ) );

	QString txt;
	txt += "<h2>";
	txt += tr ( "%1 - %2" ).arg ( PROGRAM_TITLE ).arg ( VERSION );
	txt += "</h2>";
	_title.setText ( txt );

	// Description
	txt.clear();
	txt += "<h3>";
	txt += tr ( "About" );
	txt += "</h3>\n";
	{
		txt += read_doc_file ( "desc.html" );
	}

	// Internet
	txt += "<h3>";
	txt += tr ( "Internet" );
	txt += "</h3>\n";

	txt += "<ul>";
	txt += "<li>";
	txt += "<a href=\"http://www.xwmw.org/alsamixer-qt4\">http://www.xwmw.org/alsamixer-qt4</a>";
	txt += "</li>\n";
	txt += "<li>";
	txt += "<a href=\"http://sourceforge.net/projects/alsamixer-qt4/\">http://sourceforge.net/projects/alsamixer-qt4/</a>";
	txt += "</li>\n";
	txt += "</ul>\n";

	// Authors
	txt += "<h3>";
	txt += tr ( "Authors" );
	txt += "</h3>\n";
	txt += "<p>";
	txt += "Sebastian Holtermann ";
	txt += "&lt;<a href=\"mailto:sebholt@xwmw.org\">sebholt@xwmw.org</a>&gt;";
	txt += "</p>\n";

	_txt_info.setAlignment ( Qt::AlignLeft | Qt::AlignTop );
	_txt_info.setWordWrapMode ( QTextOption::NoWrap );
	_txt_info.setOpenLinks ( true );
	_txt_info.setOpenExternalLinks ( true );
	_txt_info.setHtml ( txt );


	_txt_license.setAlignment ( Qt::AlignLeft | Qt::AlignTop );
	_txt_license.setWordWrapMode ( QTextOption::NoWrap );
	_txt_license.setOpenLinks ( true );
	_txt_license.setOpenExternalLinks ( true );

	QFont fnt ( _txt_license.document()->defaultFont() );
	fnt.setFamily ( "courier" );
	fnt.setFixedPitch ( true );
	_txt_license.document()->setDefaultFont ( fnt );

	_txt_license.setPlainText ( read_doc_file ( "COPYING" ) );

	_tabs.addTab ( &_txt_info, tr ( "Information" ) );
	_tabs.addTab ( &_txt_license, tr ( "License" ) );

	setLayout ( new QVBoxLayout ( ) );
	layout()->addWidget ( &_title );
	layout()->addWidget ( &_tabs );
}



QString
Dialog_Info::read_doc_file (
	const QString & filename_n ) const
{
	QString txt;

	QString fl_abs ( INSTALL_DIR_DOC );
	fl_abs += "/";
	fl_abs += filename_n;

	const QString fl_real ( find_l10n_file ( fl_abs ) );

	if ( !fl_real.isEmpty() ) {
		QFile file ( fl_real );
		file.open ( QIODevice::ReadOnly );
		if ( file.isOpen() ) {
			QByteArray ba = file.readAll();
			txt = QString::fromUtf8 ( ba.data(), ba.size() );
		} else {
			txt = tr ( "Error: Could not read from file %1" ).arg ( fl_real );
		}
	} else {
		txt = tr ( "Error: The file %1 does not exist" ).arg ( fl_abs );
	}


	return txt;
}



QString
Dialog_Info::find_l10n_file (
	const QString & filename_n ) const
{
	QString res;

	const QFileInfo fl_info ( filename_n );
	const QDir fl_dir ( fl_info.absoluteDir() );

	// A loop to exit with a break statement
	while ( fl_dir.exists() ) {

		const QLocale loc;
		QString fl_base ( fl_info.baseName() );
		QString fl_suff ( fl_info.completeSuffix() );

		if ( !fl_base.isEmpty() ) {
			fl_base.append ( "_" );
		}
		fl_base.prepend ( fl_dir.absolutePath() + "/" );

		if ( !fl_suff.isEmpty() ) {
			fl_suff.prepend ( "." );
		}

		QString loc_suff ( loc.name().toLower() );
		QString fl_test;

		// Test with language and country suffix
		fl_test = fl_base;
		fl_test += loc_suff;
		fl_test += fl_suff;
		if ( QFile::exists ( fl_test ) ) {
			res = fl_test;
			break;
		}

		// Test with language suffix only
		loc_suff = loc_suff.section ( "_", 0, 0 );

		fl_test = fl_base;
		fl_test += loc_suff;
		fl_test += fl_suff;
		if ( QFile::exists ( fl_test ) ) {
			res = fl_test;
			break;
		}

		// Test with default language suffix only
		loc_suff = "en";

		fl_test = fl_base;
		fl_test += loc_suff;
		fl_test += fl_suff;
		if ( QFile::exists ( fl_test ) ) {
			res = fl_test;
			break;
		}

		// Test pure file name
		fl_test = filename_n;
		if ( QFile::exists ( fl_test ) ) {
			res = fl_test;
			break;
		}

		break;
	}


	return res;
}
