//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_qmixer_window_hpp__
#define __INC_qmixer_window_hpp__

#include "config.hpp"

#include <QMainWindow>
#include <QMenuBar>
#include <QStatusBar>

#include <QDockWidget>
#include <QSplitter>
#include <QPointer>
#include <QPushButton>

#include "mixer_sliders.hpp"
#include "mixer_switches_scroll_area.hpp"
#include "side_interface.hpp"

#include "dialog_info.hpp"


class Mixer_Main_Window :
	public QMainWindow
{
	Q_OBJECT

	public:

	Mixer_Main_Window (
		QWidget * parent = 0,
		Qt::WindowFlags flags = 0 );

	~Mixer_Main_Window ( );


	QSnd::Mixer_Sliders &
	mixer_sliders ( );

	QSnd::Mixer_Switches &
	mixer_switches ( );


	const QSnd::Snd_Control_Info &
	current_ctl ( ) const;

	void
	set_current_ctl (
		const QSnd::Snd_Control_Info & ctl_n );

	void
	set_default_ctl ( );

	void
	set_mixer_config (
		const QSnd::Mixer_Config & cfg_n );


	// Protected slots
	protected slots:

	void
	show_dialog_info ( );

	void
	update_current_ctl ( );

	void
	update_mixer_config ( );


	void
	mixer_elements_changed ( );

	void
	refresh_data ( );

	void
	toggle_fullscreen ( );


	// Protected methods
	protected:

	bool
	event (
		QEvent * event_n );

	void
	closeEvent (
		QCloseEvent * event );


	// Private methods
	private:

	void
	reset_hsplitter ( );

	void
	reset_vsplitter ( );

	void
	init_menu ( );


	// Private attributes
	private:

	QSnd::Pixmaps_Hub _pixmaps_hub;

	QSnd::Mixer_Config _mixer_config;

	QSplitter _split_vert;

	QSplitter _split_hor;

	QSnd::Mixer_Sliders _mixer_sliders;

	QSnd::Mixer_Switches_Scroll_Area _mixer_switches_area;


	QSnd::Snd_Mixer_Simple _qsnd_mixer;

	QSnd::Snd_Control_Info _current_ctl;



	// Menu

	QAction * _act_exit;

	QAction * _act_refresh;

	QAction * _act_fullscreen;

	QAction * _act_info;

	QMenu * _menu_file;

	QMenu * _menu_view;

	QMenu * _menu_help;

	Side_Interface _side_iface;


	// Dialogs

	QPointer < Dialog_Info > _dialg_info;
};


inline
const QSnd::Snd_Control_Info &
Mixer_Main_Window::current_ctl ( ) const
{
	return _current_ctl;
}


inline
QSnd::Mixer_Sliders &
Mixer_Main_Window::mixer_sliders ( )
{
	return _mixer_sliders;
}


inline
QSnd::Mixer_Switches &
Mixer_Main_Window::mixer_switches ( )
{
	return _mixer_switches_area.mixer_switches();
}


#endif


