//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_mixer_sliders_hpp__
#define __INC_mixer_sliders_hpp__

#include <QString>
#include <QAction>
#include <QMenu>
#include <QTimer>

#include "mixer_config.hpp"
#include "snd_mixer_simple.hpp"
#include "pixmaps_hub.hpp"
#include "sliders_scroll_pad.hpp"

#include "mixer_proxy_selection.hpp"
#include "mixer_sliders_proxies_group.hpp"
#include "mixer_sliders_status_bar.hpp"


namespace QSnd
{


///
/// @brief Mixer_Sliders
///
class Mixer_Sliders :
	public QWidget
{
	Q_OBJECT

	// Public methods
	public:

	Mixer_Sliders (
		QWidget * parent_n = 0 );

	~Mixer_Sliders ( );


	Wdg::Sliders_Pad *
	sliders_pad ( );


	void
	set_mixer_simple (
		Snd_Mixer_Simple * mixer_n  );

	void
	set_mixer_config (
		const Mixer_Config & cfg_n );

	void
	set_pixmaps_hub (
		Pixmaps_Hub * hub_n );


	// Protected slots
	protected slots:

	void
	action_toggle_joined_separated ( );

	void
	action_level_volumes ( );

	void
	action_mute_all ( );

	void
	action_unmute_all ( );

	void
	mixer_values_changed ( );

	void
	timed_separation_check ( );


	// Protected methods
	protected:

	// Proxy creation and setup

	void
	clear_proxies_groups ( );

	void
	create_proxies_groups ( );

	void
	setup_proxies_group_joined (
		Mixer_Sliders_Proxies_Group * mspg_n );

	void
	setup_proxies_group_separate (
		Mixer_Sliders_Proxies_Group * mspg_n );

	bool
	create_proxies_set (
		Mixer_Sliders_Proxies_Group * mspg_n,
		unsigned int channel_idx_n );


	void
	update_visible_proxies_groups ( );

	void
	update_visible_mixer_values ( );

	void
	update_layout ( );


	// Focus proxy

	void
	acquire_focus_proxy (
		Mixer_Proxy_Selection & sel_n );

	void
	set_focus_proxy (
		const Mixer_Proxy_Selection & sel_n );


	void
	toggle_joined_separated (
		Mixer_Sliders_Proxies_Group * mspg_n );

	void
	proxies_group_join (
		Wdg::Sliders_Pad_Proxies_Group * sppg_n );

	void
	proxies_group_separate (
		Wdg::Sliders_Pad_Proxies_Group * sppg_n );


	// Visibility checks

	bool
	should_be_visible (
		const Mixer_Sliders_Proxies_Group * mspg_n ) const;


	// Separation checks

	bool
	should_be_separated (
		const Mixer_Sliders_Proxies_Group * mspg_n ) const;

	bool
	needs_separation (
		const Mixer_Sliders_Proxies_Group * mspg_n ) const;


	void
	evaluate_mixer_values ( );


	// Context menu

	void
	context_menu_start (
		const QPoint & pos_n );

	void
	context_menu_proxies_group (
		Mixer_Sliders_Proxies_Group * * mspg_n,
		Mixer_Sliders_Proxies_Set * * msps_n = 0 );

	void
	context_menu_update ( );



	/// @brief Convenience method that does a static cast
	const Mixer_Sliders_Proxies_Group *
	proxies_group (
		const Wdg::Sliders_Pad_Proxies_Group * grp_n ) const;

	Mixer_Sliders_Proxies_Group *
	proxies_group (
		Wdg::Sliders_Pad_Proxies_Group * grp_n );


	bool
	eventFilter (
		QObject * watched,
		QEvent * event );


	// Private attributes
	private:

	Mixer_Sliders_Proxies_Group_List _proxies_groups;

	Wdg::Sliders_Pad_Proxies_Group_List _proxies_groups_visible;


	Pixmaps_Hub * _pixmaps_hub;

	Wdg::Sliders_Scroll_Pad _sliders_scroll_pad;

	QFrame _status_bar_frame;

	Mixer_Sliders_Status_Bar _status_bar;


	Mixer_Proxy_Selection _context_menu_focus;

	QMenu _context_menu;

	unsigned int _actions_visible;

	QAction _act_separate_join;

	QAction _act_level_volumes;

	QAction _act_separator_channels;

	QAction _act_mute_all;

	QAction _act_unmute_all;


	QString _act_str_toggle_separate[2];

	QString _act_str_mute[2];

	QString _act_str_unmute[2];


	QTimer _timer_separation;


	Mixer_Config _config;

	Snd_Mixer_Simple * _mixer_simple;
};


inline
Wdg::Sliders_Pad *
Mixer_Sliders::sliders_pad ( )
{
	return &_sliders_scroll_pad.sliders_pad();
}


inline
Mixer_Sliders_Proxies_Group *
Mixer_Sliders::proxies_group (
	Wdg::Sliders_Pad_Proxies_Group * grp_n )
{
	return static_cast < Mixer_Sliders_Proxies_Group * > ( grp_n );
}


inline
const Mixer_Sliders_Proxies_Group *
Mixer_Sliders::proxies_group (
	const Wdg::Sliders_Pad_Proxies_Group * grp_n ) const
{
	return static_cast < const Mixer_Sliders_Proxies_Group * > ( grp_n );
}


inline
bool
Mixer_Sliders::should_be_separated (
	const Mixer_Sliders_Proxies_Group * mspg_n ) const
{
	const Snd_Mixer_Simple_Elem * qsme ( mspg_n->mixer_simple_elem() );

	bool res (
		qsme->volumes_equal ( mspg_n->snd_dir() ) &&
		qsme->switches_equal ( mspg_n->snd_dir() ) );
	res = !res;
	return res;
}


inline
bool
Mixer_Sliders::needs_separation (
	const Mixer_Sliders_Proxies_Group * mspg_n ) const
{
	bool res = ( mspg_n->is_joined() && should_be_separated ( mspg_n ) );
	return res;
}


inline
bool
Mixer_Sliders::should_be_visible (
	const Mixer_Sliders_Proxies_Group * mspg_n ) const
{
	const Snd_Mixer_Simple_Elem * qsme ( mspg_n->mixer_simple_elem() );

	bool res ( false );
	if ( qsme->is_active() ) {
		res = res || ( _config.show_dir[0] && ( mspg_n->snd_dir() == 0 ) );
		res = res || ( _config.show_dir[1] && ( mspg_n->snd_dir() == 1 ) );
	}
	return res;
}


} // End of namespace


#endif


