//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_mixer_sliders_proxies_group_hpp__
#define __INC_mixer_sliders_proxies_group_hpp__

#include <QString>
#include <QTime>
#include <QList>

#include "sliders_pad_proxies_group.hpp"
#include "mixer_sliders_proxies_set.hpp"
#include "mixer_separation_info.hpp"


namespace QSnd
{


///
/// @brief Mixer_Sliders_Proxies_Group
///
class Mixer_Sliders_Proxies_Group :
	public Wdg::Sliders_Pad_Proxies_Group
{
	Q_OBJECT

	// Public methods
	public:

	Mixer_Sliders_Proxies_Group (
		QObject * parent_n = 0 );

	~Mixer_Sliders_Proxies_Group ( );


	// Snd_Mixer_Simple_Elem

	Snd_Mixer_Simple_Elem *
	mixer_simple_elem ( ) const;

	virtual
	void
	set_mixer_simple_elem (
		Snd_Mixer_Simple_Elem * selem_n );


	// Snd dir

	unsigned char
	snd_dir ( ) const;

	void
	set_snd_dir (
		unsigned char dir_n );


	// Joined flag

	bool
	is_joined ( ) const;

	void
	set_is_joined (
		bool flag_n );

	bool
	can_be_separate ( ) const;


	// If true then the corresponding widget is visible

	bool
	is_visible ( ) const;

	void
	set_is_visible (
		bool flag_n );


	// Separation info

	const Mixer_Separation_Info &
	separation_info ( ) const;

	Mixer_Separation_Info &
	separation_info ( );


	// Value updating

	void
	update_mixer_values ( );


	// Protected methods
	protected:


	// Private attributes
	private:

	Snd_Mixer_Simple_Elem * _mixer_simple_elem;

	unsigned char _snd_dir;

	bool _is_joined;

	bool _is_visible;

	Mixer_Separation_Info _separation_info;
};


inline
Snd_Mixer_Simple_Elem *
Mixer_Sliders_Proxies_Group::mixer_simple_elem ( ) const
{
	return _mixer_simple_elem;
}


inline
unsigned char
Mixer_Sliders_Proxies_Group::snd_dir ( ) const
{
	return _snd_dir;
}


inline
bool
Mixer_Sliders_Proxies_Group::is_joined ( ) const
{
	return _is_joined;
}


inline
bool
Mixer_Sliders_Proxies_Group::can_be_separate ( ) const
{
	if ( mixer_simple_elem() != 0 ) {
		return ( mixer_simple_elem()->num_channels ( snd_dir() ) > 1 );
	}
	return false;
}


inline
bool
Mixer_Sliders_Proxies_Group::is_visible ( ) const
{
	return _is_visible;
}


inline
void
Mixer_Sliders_Proxies_Group::set_is_visible (
	bool flag_n )
{
	_is_visible = flag_n;
}


inline
const Mixer_Separation_Info &
Mixer_Sliders_Proxies_Group::separation_info ( ) const
{
	return _separation_info;
}


inline
Mixer_Separation_Info &
Mixer_Sliders_Proxies_Group::separation_info ( )
{
	return _separation_info;
}


typedef QList < Mixer_Sliders_Proxies_Group * > Mixer_Sliders_Proxies_Group_List;


} // End of namespace


#endif


