//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "mixer_sliders_proxy_slider.hpp"

#include <QApplication>
#include <iostream>


namespace QSnd
{


//
// Mixer_Sliders_Proxy_Slider
//

Mixer_Sliders_Proxy_Slider::Mixer_Sliders_Proxy_Slider (
	QObject * parent_n ) :
Wdg::Sliders_Pad_Proxy_Slider ( parent_n ),
_mixer_simple_elem ( 0 ),
_dB_val ( 0 ),
_channel_idx ( 0 ),
_snd_dir ( 0 ),
_alsa_updating ( false )
{
	update_buffers();
}


void
Mixer_Sliders_Proxy_Slider::set_snd_dir (
	unsigned char dir_n )
{
	_snd_dir = dir_n;
	update_buffers();
}


void
Mixer_Sliders_Proxy_Slider::set_channel_idx (
	unsigned int idx_n )
{
	_channel_idx = idx_n;
}


void
Mixer_Sliders_Proxy_Slider::set_mixer_simple_elem (
	Snd_Mixer_Simple_Elem * selem_n )
{
	_mixer_simple_elem = selem_n;
	update_buffers();
}


void
Mixer_Sliders_Proxy_Slider::set_is_joined (
	bool flag_n )
{
	_is_joined = flag_n;
}


void
Mixer_Sliders_Proxy_Slider::set_dB_value (
	long dB_val_n )
{
	if ( has_dB() ) {
		if ( _dB_val != dB_val_n ) {
			_dB_val = dB_val_n;
			this->dB_value_changed();
			emit sig_dB_value_changed ( _dB_val );
		}
	}
}


void
Mixer_Sliders_Proxy_Slider::update_buffers ( )
{
	if ( mixer_simple_elem() != 0 ) {
		_channel_buffer = &mixer_simple_elem()->channel_buffer ( snd_dir() );
		_volume_buffer = &mixer_simple_elem()->volume_buffer ( snd_dir() );
		_dB_buffer = &mixer_simple_elem()->dB_buffer ( snd_dir() );
	} else {
		_channel_buffer = 0;
		_volume_buffer = 0;
		_dB_buffer = 0;
	}
}


void
Mixer_Sliders_Proxy_Slider::dB_value_changed ( )
{
	//std::cout << "Mixer_Sliders_Proxy_Slider::dB_value_changed " << slider_value() << "\n";

	if ( ( mixer_simple_elem() != 0 ) && ( !_alsa_updating ) ) {
		if ( is_joined() ) {
			mixer_simple_elem()->set_dB_all ( snd_dir(), dB_value() );
		} else {
			mixer_simple_elem()->set_dB ( snd_dir(), channel_idx(), dB_value() );
		}
	}

	//std::cout << "Mixer_Sliders_Proxy_Slider::dB_value_changed " << "done" << "\n";
}


void
Mixer_Sliders_Proxy_Slider::slider_value_changed ( )
{
	//std::cout << "Mixer_Sliders_Proxy_Slider::slider_value_changed " << slider_value() << "\n";

	if ( ( mixer_simple_elem() != 0 ) && ( !_alsa_updating ) ) {
		if ( is_joined() || ( QApplication::keyboardModifiers() & Qt::ControlModifier ) ) {
			mixer_simple_elem()->set_volume_all ( snd_dir(), slider_value() );
		} else {
			mixer_simple_elem()->set_volume ( snd_dir(), channel_idx(), slider_value() );
		}
	}

	//std::cout << "Mixer_Sliders_Proxy_Slider::slider_value_changed " << "done" << "\n";
}


void
Mixer_Sliders_Proxy_Slider::update_mixer_values ( )
{
	if ( ( mixer_simple_elem() != 0 ) && ( !_alsa_updating ) ) {
		_alsa_updating = true;

		if ( has_dB() ) {
			set_dB_value ( (*_dB_buffer)[channel_idx()] );
		}
		set_slider_value ( (*_volume_buffer)[channel_idx()] );

		_alsa_updating = false;
	}
}


} // End of namespace

