//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_mixer_switches_hpp__
#define __INC_mixer_switches_hpp__

#include "snd_mixer_simple.hpp"
#include "pixmaps_hub.hpp"
#include "mixer_config.hpp"
#include "mixer_proxy_selection.hpp"
#include "mixer_switches_block.hpp"
#include "mixer_switches_proxies_group.hpp"

#include <QWidget>
#include <QCheckBox>
#include <QComboBox>
#include <QMenu>
#include <QTimer>

#include <QList>


namespace QSnd
{


///
/// @brief Mixer_Switches
///
class Mixer_Switches :
	public QWidget
{
	Q_OBJECT

	// Public methods
	public:

	Mixer_Switches (
		QWidget * parent_n = 0 );

	~Mixer_Switches ( );


	void
	set_mixer_simple (
		Snd_Mixer_Simple * mixer_n );

	void
	set_mixer_config (
		const Mixer_Config & cfg_n );


	bool
	has_switches (
		int snd_dir_n ) const;

	bool
	has_visible_blocks ( ) const;


	// Public signals
	signals:

	void
	sig_layout_about_to_change ( );

	void
	sig_layout_changed ( );


	// Protected slots
	protected slots:

	void
	action_toggle_joined_separated ( );

	void
	mixer_values_changed ( );

	void
	timed_separation_check ( );


	// Protected methods
	protected:

	void
	clear_proxies_groups ( );

	void
	create_proxies_groups ( );

	void
	setup_proxies_group_joined (
		Mixer_Switches_Proxies_Group * mspg_n );

	void
	setup_proxies_group_separate (
		Mixer_Switches_Proxies_Group * mspg_n );

	Mixer_Switches_Proxy *
	create_proxy (
		Mixer_Switches_Proxies_Group * mspg_n,
		int channel_idx_n );


	void
	update_visible_proxies_group (
		int snd_dir_n );

	void
	update_visible_proxies_groups ( );

	void
	update_layout ( );

	void
	update_visible_mixer_values ( );



	void
	toggle_joined_separated (
		Wdg::Switches_Pad * pad_n,
		Mixer_Switches_Proxies_Group * mspg_n );

	void
	proxies_group_join (
		Mixer_Switches_Proxies_Group * mspg_n );

	void
	proxies_group_separate (
		Mixer_Switches_Proxies_Group * mspg_n );


	bool
	should_be_separated (
		const Mixer_Switches_Proxies_Group * mspg_n ) const;

	bool
	needs_separation (
		const Mixer_Switches_Proxies_Group * mspg_n ) const;

	bool
	should_be_visible (
		const Mixer_Switches_Proxies_Group * mspg_n ) const;


	void
	evaluate_mixer_values ( );


	Wdg::Switches_Pad *
	acquire_focus_pad ( );

	void
	acquire_focus_proxy (
		Wdg::Switches_Pad * sw_pad_n,
		Mixer_Proxy_Selection & sel_n );

	void
	set_focus_proxy (
		Wdg::Switches_Pad * sw_pad_n,
		const Mixer_Proxy_Selection & sel_n );


	void
	context_menu_start (
		Wdg::Switches_Pad * sw_pad_n,
		const QPoint & pos_n );

	Mixer_Switches_Proxies_Group *
	context_menu_proxies_group (
		Wdg::Switches_Pad * sw_pad_n );

	void
	context_menu_update ( );


	bool
	eventFilter (
		QObject * watched,
		QEvent * event );


	/// @brief Convenience method that does a type cast
	const Mixer_Switches_Proxies_Group *
	proxies_group (
		const Wdg::Switches_Pad_Proxies_Group * grp_n ) const;

	Mixer_Switches_Proxies_Group *
	proxies_group (
		Wdg::Switches_Pad_Proxies_Group * grp_n );

	Wdg::Switches_Pad *
	switches_pad (
		int snd_dir_n );


	// Private attributes
	private:

	Pixmaps_Hub * _pixmaps_hub;

	Mixer_Switches_Block _sw_block[2];

	Mixer_Switches_Proxy_Group_List _proxies_groups;
	Mixer_Switches_Proxy_Group_List _proxies_groups_dir[2];
	Wdg::Switches_Pad_Proxy_Group_List _proxies_groups_visible[2];


	Mixer_Proxy_Selection _context_menu_focus;

	QMenu _context_menu;

	unsigned int _actions_visible;

	QAction _act_separate_join;

	QString _act_str_separate_join[2];


	QTimer _timer_separation;


	Mixer_Config _config;

	Snd_Mixer_Simple * _mixer;
};


inline
const Mixer_Switches_Proxies_Group *
Mixer_Switches::proxies_group (
	const Wdg::Switches_Pad_Proxies_Group * grp_n ) const
{
	return static_cast < const Mixer_Switches_Proxies_Group * > ( grp_n );
}


inline
Mixer_Switches_Proxies_Group *
Mixer_Switches::proxies_group (
	Wdg::Switches_Pad_Proxies_Group * grp_n )
{
	return static_cast < Mixer_Switches_Proxies_Group * > ( grp_n );
}


inline
bool
Mixer_Switches::has_switches (
	int snd_dir_n ) const
{
	return ( _proxies_groups_visible[snd_dir_n].size() > 0 );
}


inline
bool
Mixer_Switches::should_be_separated (
	const Mixer_Switches_Proxies_Group * mspg_n ) const
{
	const Snd_Mixer_Simple_Elem * qsme ( mspg_n->mixer_simple_elem() );

	bool res (
		qsme->switches_equal ( mspg_n->snd_dir() ) &&
		qsme->enums_equal ( mspg_n->snd_dir() ) );
	res = !res;
	return res;
}


inline
bool
Mixer_Switches::needs_separation (
	const Mixer_Switches_Proxies_Group * mspg_n ) const
{
	bool res ( mspg_n->is_joined() && should_be_separated ( mspg_n ) );
	return res;
}


inline
bool
Mixer_Switches::should_be_visible (
	const Mixer_Switches_Proxies_Group * mspg_n ) const
{
	const Snd_Mixer_Simple_Elem * qsme ( mspg_n->mixer_simple_elem() );
	return qsme->is_active();
}


inline
Wdg::Switches_Pad *
Mixer_Switches::switches_pad (
	int snd_dir_n )
{
	return &_sw_block[snd_dir_n].switches_pad();
}


} // End of namespace


#endif

