//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_mixer_switches_proxy_group_hpp__
#define __INC_mixer_switches_proxy_group_hpp__

#include <QObject>
#include <QString>
#include <QList>

#include "switches_pad_proxies_group.hpp"
#include "mixer_switches_proxy.hpp"
#include "mixer_separation_info.hpp"


namespace QSnd
{


///
/// @brief Mixer_Switches_Proxies_Group
///
class Mixer_Switches_Proxies_Group :
	public Wdg::Switches_Pad_Proxies_Group
{
	Q_OBJECT

	// Public typedefs
	public:

	typedef QList < Mixer_Switches_Proxy * > Proxy_List;


	// Public methods
	public:

	Mixer_Switches_Proxies_Group (
		QObject * parent_n = 0 );

	~Mixer_Switches_Proxies_Group ( );


	Mixer_Switches_Proxy *
	mixer_proxy (
		int idx_n );


	// Snd_Mixer_Simple_Elem

	Snd_Mixer_Simple_Elem *
	mixer_simple_elem ( ) const;

	virtual
	void
	set_mixer_simple_elem (
		Snd_Mixer_Simple_Elem * selem_n );


	// Snd direction

	unsigned char
	snd_dir ( ) const;

	void
	set_snd_dir (
		unsigned char dir_n );


	// Is joined flag

	bool
	is_joined ( ) const;

	void
	set_is_joined (
		bool flag_n );


	bool
	can_be_separate ( ) const;


	// Is visible flag

	bool
	is_visible ( ) const;

	void
	set_is_visible (
		bool flag_n );



	// Separation info

	const Mixer_Separation_Info &
	separation_info ( ) const;

	Mixer_Separation_Info &
	separation_info ( );



	// Update values

	void
	update_mixer_values ( );


	// Private attributes
	private:

	Snd_Mixer_Simple_Elem * _mixer_simple_elem;

	unsigned char _snd_dir;

	bool _is_joined;

	bool _is_visible;

	Mixer_Separation_Info _separation_info;

};


inline
Mixer_Switches_Proxy *
Mixer_Switches_Proxies_Group::mixer_proxy (
	int idx_n )
{
	return static_cast < Mixer_Switches_Proxy * > ( proxy ( idx_n ) );
}


inline
Snd_Mixer_Simple_Elem *
Mixer_Switches_Proxies_Group::mixer_simple_elem ( ) const
{
	return _mixer_simple_elem;
}


inline
unsigned char
Mixer_Switches_Proxies_Group::snd_dir ( ) const
{
	return _snd_dir;
}


inline
bool
Mixer_Switches_Proxies_Group::is_joined ( ) const
{
	return _is_joined;
}


inline
bool
Mixer_Switches_Proxies_Group::is_visible ( ) const
{
	return _is_visible;
}


inline
bool
Mixer_Switches_Proxies_Group::can_be_separate ( ) const
{
	if ( mixer_simple_elem() != 0 ) {
		return ( mixer_simple_elem()->num_channels ( snd_dir() ) > 1 );
	}
	return false;
}


inline
const Mixer_Separation_Info &
Mixer_Switches_Proxies_Group::separation_info ( ) const
{
	return _separation_info;
}


inline
Mixer_Separation_Info &
Mixer_Switches_Proxies_Group::separation_info ( )
{
	return _separation_info;
}


typedef QList < Mixer_Switches_Proxies_Group * > Mixer_Switches_Proxy_Group_List;


} // End of namespace


#endif


