//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "mixer_switches_proxy.hpp"

#include <iostream>


namespace QSnd
{


//
// Mixer_Switches_Proxy
//

Mixer_Switches_Proxy::Mixer_Switches_Proxy (
	QObject * parent_n ) :
Wdg::Switches_Pad_Proxy ( parent_n ),
_mixer_simple_elem ( 0 ),
_channel_idx ( 0 ),
_snd_dir ( 0 ),
_is_joined ( false ),
_alsa_updating ( false )
{
	update_buffers();
}


void
Mixer_Switches_Proxy::set_snd_dir (
	unsigned char dir_n )
{
	_snd_dir = dir_n;
	update_buffers();
}


void
Mixer_Switches_Proxy::set_channel_idx (
	unsigned int idx_n )
{
	_channel_idx = idx_n;
}


void
Mixer_Switches_Proxy::set_mixer_simple_elem (
	Snd_Mixer_Simple_Elem * selem_n )
{
	_mixer_simple_elem = selem_n;

	update_buffers();
}


void
Mixer_Switches_Proxy::set_is_joined (
	bool flag_n )
{
	_is_joined = flag_n;
}


void
Mixer_Switches_Proxy::update_buffers ( )
{
	if ( mixer_simple_elem() != 0 ) {
		_channel_buffer = &mixer_simple_elem()->channel_buffer ( snd_dir() );
		_switch_buffer = &mixer_simple_elem()->switch_buffer ( snd_dir() );
		_enum_buffer = &mixer_simple_elem()->enum_buffer ( snd_dir() );
	} else {
		_channel_buffer = 0;
		_switch_buffer = 0;
		_enum_buffer = 0;
	}
}


QString
Mixer_Switches_Proxy::enum_item_name (
	int idx_n )
{
	QString res;
	if ( mixer_simple_elem() != 0 ) {
		if ( static_cast < unsigned int > ( idx_n ) <
			mixer_simple_elem()->enum_item_names().size() )
		{
			res = mixer_simple_elem()->enum_item_names()[idx_n];
		}
	}
	return res;
}


void
Mixer_Switches_Proxy::switch_state_changed ( )
{
	if ( ( mixer_simple_elem() != 0 ) && ( !_alsa_updating ) ) {
		//std::cout << "Mixer_Switches_Proxy::switch_state_changed " << switch_state() << "\n";

		if ( is_joined() ) {
			mixer_simple_elem()->set_switch_all ( snd_dir(), switch_state() );
		} else {
			mixer_simple_elem()->set_switch ( snd_dir(), channel_idx(), switch_state() );
		}

		//std::cout << "Mixer_Switches_Proxy::switch_state_changed " << "done" << "\n";
	}
}


void
Mixer_Switches_Proxy::enum_index_changed ( )
{
	if ( ( mixer_simple_elem() != 0 ) && ( !_alsa_updating ) ) {
		//std::cout << "Mixer_Switches_Proxy::enum_index_changed " << enum_index() << "\n";

		if ( is_joined() ) {
			mixer_simple_elem()->set_enum_index_all ( snd_dir(), enum_index() );
		} else {
			mixer_simple_elem()->set_enum_index ( snd_dir(), channel_idx(), enum_index() );
		}

		//std::cout << "Mixer_Switches_Proxy::enum_index_changed " << "done" << "\n";
	}
}


void
Mixer_Switches_Proxy::update_mixer_values ( )
{
	if ( ( mixer_simple_elem() != 0 ) && ( !_alsa_updating ) ) {

		_alsa_updating = true;

		//std::cout << "Mixer_Switches_Proxy::update_mixer_values " << "\n";

		if ( is_switch() ) {
			set_switch_state ( (*_switch_buffer)[channel_idx()] );
		} else if ( is_enum() ) {
			set_enum_index ( (*_enum_buffer)[channel_idx()] );
		}

		_alsa_updating = false;
	}
}


} // End of namespace
