//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "mixer_switches_scroll_area.hpp"

#include <iostream>


namespace QSnd
{


//
// Mixer_Switches_Scroll_Area
//
Mixer_Switches_Scroll_Area::Mixer_Switches_Scroll_Area (
	QWidget * parent_n ) :
QScrollArea ( parent_n ),
_mixer_switches ( this )
{
	{
		QSizePolicy policy ( sizePolicy() );
		policy.setHorizontalPolicy ( QSizePolicy::Minimum );
		policy.setVerticalPolicy ( QSizePolicy::Maximum );
		setSizePolicy ( policy );
	}

	// Setup scrollarea
	setHorizontalScrollBarPolicy ( Qt::ScrollBarAlwaysOff );
	setFrameStyle ( QFrame::NoFrame );
	setWidgetResizable ( true );
	setWidget ( &_mixer_switches );

	connect ( &_mixer_switches, SIGNAL ( sig_layout_about_to_change() ),
		this, SLOT ( layout_about_to_change() ) );

	connect ( &_mixer_switches, SIGNAL ( sig_layout_changed() ),
		this, SLOT ( layout_changed() ) );
}


Mixer_Switches_Scroll_Area::~Mixer_Switches_Scroll_Area ( )
{
}


QSize
Mixer_Switches_Scroll_Area::sizeHint ( ) const
{
	ensurePolished();

	QSize res ( _mixer_switches.sizeHint() );
	int & ww ( res.rwidth() );
	int & hh ( res.rheight() );

	const QSize vsbsh ( verticalScrollBar()->sizeHint() );
	ww += vsbsh.width();
	ww += 2*frameWidth();
	hh += 2*frameWidth();

	return res;
}


QSize
Mixer_Switches_Scroll_Area::minimumSizeHint ( ) const
{
	ensurePolished();

	QSize res ( 0, 0 );
	int & ww ( res.rwidth() );
	int & hh ( res.rheight() );

	const QSize vsbsh ( verticalScrollBar()->sizeHint() );
	hh += vsbsh.height();
	ww += vsbsh.width();

	const QSize mssh ( _mixer_switches.minimumSizeHint() );
	ww += mssh.width();

	hh += 2*frameWidth();
	ww += 2*frameWidth();

	return res;
}


void
Mixer_Switches_Scroll_Area::layout_about_to_change ( )
{
	//std::cout << "Mixer_Switches_Scroll_Area::layout_about_to_change" << "\n";
	_mixer_switches.hide();
}


void
Mixer_Switches_Scroll_Area::layout_changed ( )
{
	//std::cout << "Mixer_Switches_Scroll_Area::layout_changed" << "\n";
	if ( _mixer_switches.has_visible_blocks() ) {
		_mixer_switches.show();
		updateGeometry();
		setMaximumHeight ( sizeHint().height() );
		show();
	} else {
		hide();
	}
}


} // End of namespace

