//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "pixmaps_hub.hpp"

#include "qslider_painter_box.hpp"
#include "qcheck_box_painter_box.hpp"
#include <QApplication>
#include <QPalette>


namespace QSnd
{


Pixmaps_Hub::Pixmaps_Hub ( ) :
_painter_cb ( 0 ),
_painter_slider ( 0 )
{
	_cb_buffers.set_buff_bg ( new SPix::Shared_Pixmap_Set_Buffer ( 0 ) );
	_cb_buffers.set_buff_handle ( new SPix::Shared_Pixmap_Set_Buffer ( 1 ) );

	_slider_buffers.set_buff_bg ( new SPix::Shared_Pixmap_Set_Buffer ( 0 ) );
	_slider_buffers.set_buff_handle ( new SPix::Shared_Pixmap_Set_Buffer ( 1 ) );


	QPalette pal ( QApplication::palette() );
	QColor col_norm ( pal.color ( QPalette::Button ) );
	QColor col_high ( pal.color ( QPalette::WindowText ) );

	col_norm = Wdg::Painter::col_mix ( col_norm, col_high, 1, 1 );

	// Playback style
	_style_play.style_id = 1;
	_style_play.border_col_normal = col_norm;
	_style_play.border_col_focus = col_high;

	_style_play.col_fg = QColor ( 0, 0, 60 );
	_style_play.col_bg = QColor ( 0, 0, 240 );

	_style_play.tick_col[0] = style_play().col_fg;
	_style_play.tick_col[1] = style_play().col_fg;
	_style_play.tick_col[2] = style_play().col_fg;

	_style_play.col_active_glow = QColor ( 255, 255, 180 );
	_style_play.col_active_light = QColor ( 220, 220, 0 );
	_style_play.col_active_dark = QColor ( 200, 120, 0 );


	// Capture style
	_style_cap = _style_play;
	_style_cap.style_id = 1;
	_style_cap.col_fg = QColor ( 130, 0, 0 );
	_style_cap.col_bg = QColor ( 215, 0, 0 );

	_style_cap.tick_col[0] = style_cap().col_fg;
	_style_cap.tick_col[1] = style_cap().col_fg;
	_style_cap.tick_col[2] = style_cap().col_fg;


	// Set painters
	set_cb_painter ( new Wdg::Painter::QCheck_Box_Painter_Box() );
	set_slider_painter ( new Wdg::Painter::QSlider_Painter_Box() );
}


void
Pixmaps_Hub::set_cb_painter (
	Wdg::Painter::QCheck_Box_Painter * painter_n )
{
	if ( _painter_cb != 0 ) {
		_cb_buffers.set_painter ( 0 );
		delete _painter_cb;
	}

	_painter_cb = painter_n;

	if ( _painter_cb != 0 ) {
		_painter_cb->set_style ( 0, &style_play() );
		_painter_cb->set_style ( 1, &style_cap() );

		_cb_buffers.set_painter ( _painter_cb );
	}
}


void
Pixmaps_Hub::set_slider_painter (
	Wdg::Painter::QSlider_Painter * painter_n )
{
	if ( _painter_slider != 0 ) {
		_slider_buffers.set_painter ( 0 );
		delete _painter_slider;
	}

	_painter_slider = painter_n;

	if ( _painter_slider != 0 ) {
		_painter_slider->set_style ( 0, &style_play() );
		_painter_slider->set_style ( 1, &style_cap() );

		_slider_buffers.set_painter ( _painter_slider );
	}

}


Pixmaps_Hub::~Pixmaps_Hub ( )
{
	set_cb_painter ( 0 );
	set_slider_painter ( 0 );

	delete _slider_buffers.buff_bg();
	delete _slider_buffers.buff_handle();

	delete _cb_buffers.buff_bg();
	delete _cb_buffers.buff_handle();
}


} // End of namespace

