//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "qcheck_box.hpp"

#include <QPainter>


namespace Wdg {


//
// QCheck_Box
//
QCheck_Box::QCheck_Box (
	QWidget * parent,
	const QCheck_Box_Pixmaps_Buffers & buff_n,
	const QString & txt_n ) :
QAbstractButton ( parent ),
_pixmaps_bg ( 0 ),
_pixmaps_handle ( 0 ),
_pixmaps_buffers ( buff_n ),
_update_pixmaps ( true )
{
	setText ( txt_n );
	setCheckable ( true );
	{
		QSizePolicy policy ( sizePolicy() );
		policy.setHorizontalPolicy ( QSizePolicy::Preferred );
		policy.setVerticalPolicy ( QSizePolicy::Preferred );
		setSizePolicy ( policy );
	}
}


QCheck_Box::~QCheck_Box ( )
{
	set_pixmaps_buffers ( QCheck_Box_Pixmaps_Buffers() );
}


void
QCheck_Box::set_pixmaps_buffers (
	const QCheck_Box_Pixmaps_Buffers & buff_n )
{
	if ( pixmaps_buffers().buff_bg() != 0 ) {
		pixmaps_buffers().buff_bg()->return_pixmaps ( _pixmaps_bg );
	}
	if ( pixmaps_buffers().buff_handle() != 0 ) {
		pixmaps_buffers().buff_handle()->return_pixmaps ( _pixmaps_handle );
	}

	_pixmaps_buffers = buff_n;

	if ( ( pixmaps_buffers().buff_bg() != 0 ) ||
		( pixmaps_buffers().buff_handle() != 0 ) )
	{
		_update_pixmaps = true;
	}
	update();
}


QSize
QCheck_Box::sizeHint ( ) const
{
	int def_len ( fontMetrics().height() );
	return QSize ( def_len, def_len );
}


QSize
QCheck_Box::minimumSizeHint ( ) const
{
	int def_len ( fontMetrics().height() );
	return QSize ( def_len, def_len );
}


void
QCheck_Box::set_pixmaps_config (
	const QCheck_Box_Pixmaps_Config & cfg_n )
{
	_pixmaps_config = cfg_n;
	_update_pixmaps = true;
	update();
}



// Events

void
QCheck_Box::enterEvent (
	QEvent * )
{
	update();
}


void
QCheck_Box::leaveEvent (
	QEvent * )
{
	update();
}


void
QCheck_Box::resizeEvent (
	QResizeEvent * )
{
	_update_pixmaps = true;
}


void
QCheck_Box::paintEvent (
	QPaintEvent * )
{
	if ( _update_pixmaps ) {
		_update_pixmaps = false;
		update_pixmaps();
	}

	QPainter painter ( this );

	// Draw background
	if ( _pixmaps_bg != 0 ) {
		int pm_idx ( 0 );
		if ( hasFocus() ) {
			pm_idx = 1;
		}
		if ( underMouse() ) {
			pm_idx += 2;
		}
		painter.drawPixmap (
			QPoint ( 0, 0 ),
			_pixmaps_bg->pixmap ( pm_idx ) );
	}

	// Draw handle
	if ( ( _pixmaps_handle != 0 ) && isChecked() ) {
		painter.drawPixmap (
			QPoint ( 0, 0 ),
			_pixmaps_handle->pixmap ( 0 ) );
	}

}


void
QCheck_Box::update_pixmaps ( )
{
	_pixmaps_config.bg.size = size();
	_pixmaps_config.handle.size = size();

	if ( pixmaps_buffers().buff_bg() != 0 ) {
		_pixmaps_bg = pixmaps_buffers().buff_bg()->return_acquire (
			_pixmaps_bg, _pixmaps_config.bg );
	}

	if ( pixmaps_buffers().buff_handle() != 0 ) {
		_pixmaps_handle = pixmaps_buffers().buff_handle()->return_acquire (
			_pixmaps_handle, _pixmaps_config.handle );
	}
}


} // End of namespace

