//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_qcheck_box_painter_hpp__
#define __INC_qcheck_box_painter_hpp__

#include "qcheck_box_pixmaps_config.hpp"
#include "qwidget_painter.hpp"


namespace Wdg
{
namespace Painter
{



///
/// @brief QCheck_Box_Painter
///
class QCheck_Box_Painter :
	public QWidget_Painter
{

	// Public methods
	public:

	QCheck_Box_Painter ( );

	const SPix::Shared_Pixmap_Set_Config *
	pm_cfg ( ) const;


	int
	paint_pixmaps (
		SPix::Shared_Pixmap_Set * spms_n,
		int image_type_n );



	// Protected methods
	protected:

	///
	/// @brief Paints the background
	///
	/// 0 - idle
	/// 1 - focus
	/// 2 - hover idle
	/// 3 - hover focus
	///
	virtual
	int
	paint_bg (
		SPix::Shared_Pixmap_Set * spms_n ) = 0;


	///
	/// @brief Paints the handle
	///
	/// 0 - idle
	/// 1 - focus
	/// 2 - hover idle
	/// 3 - hover focus
	///
	virtual
	int
	paint_handle (
		SPix::Shared_Pixmap_Set * spms_n ) = 0;


	bool
	has_focus ( ) const;

	bool
	mouse_over ( ) const;

	void
	set_focus_mover (
		bool has_focus_n,
		bool mouse_over_n );




	// Private attributes
	private:

	const SPix::Shared_Pixmap_Set_Config * _pm_cfg;

	bool _has_focus;
	bool _mouse_over;
};


inline
const SPix::Shared_Pixmap_Set_Config *
QCheck_Box_Painter::pm_cfg ( ) const
{
	return _pm_cfg;
}


inline
bool
QCheck_Box_Painter::has_focus ( ) const
{
	return _has_focus;
}


inline
bool
QCheck_Box_Painter::mouse_over ( ) const
{
	return _mouse_over;
}


inline
void
QCheck_Box_Painter::set_focus_mover (
	bool has_focus_n,
	bool mouse_over_n )
{
	_has_focus = has_focus_n;
	_mouse_over = mouse_over_n;
}



} // End of namespace
} // End of namespace


#endif


