//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "qcheck_box_painter_box.hpp"

#include <cmath>


namespace Wdg
{
namespace Painter
{


QCheck_Box_Painter_Box::QCheck_Box_Painter_Box ( )
{
}


int
QCheck_Box_Painter_Box::paint_bg (
	SPix::Shared_Pixmap_Set * spms_n )
{
	if ( spms_n->pixmap_size().isValid() ) {
		set_is ( spms_n->pixmap_size() );

		_min_len = qMin ( is().width(), is().height() );
		_max_len = qMax ( is().width(), is().height() );
		_border_width = qMax ( _min_len / 16, 1 );

		set_focus_mover ( false, false );
		paint_bg_type ( spms_n, 0 );

		set_focus_mover ( true, false );
		paint_bg_type ( spms_n, 1 );

		set_focus_mover ( false, true );
		paint_bg_type ( spms_n, 2 );

		set_focus_mover ( true, true );
		paint_bg_type ( spms_n, 3 );

		return 0;
	}
	return -1;
}


void
QCheck_Box_Painter_Box::paint_bg_type (
	SPix::Shared_Pixmap_Set * spms_n,
	int pixmap_idx_n )
{
	QImage img ( is(), QImage::Format_ARGB32 );
	img.fill ( 0 );

	{
		QPainter pnt ( &img );
		pnt.setRenderHints (
			QPainter::Antialiasing |
			QPainter::TextAntialiasing |
			QPainter::SmoothPixmapTransform );

		set_painter ( &pnt );

		// Paint
		paint_bg_area();
		paint_bg_frame();
	}

	spms_n->pixmap_ref ( pixmap_idx_n ) = QPixmap::fromImage ( img );
}


void
QCheck_Box_Painter_Box::paint_bg_area ( )
{
	QWidget_Style_Painter spainter ( painter() );
	const QRect img_rect ( QPoint ( 0, 0 ), is() );

	const qreal bw ( _border_width );
	const qreal bevel ( _border_width );

	// Border
	{
		QColor col_border ( cstyle()->border_col_normal );
		if ( has_focus() ) {
			col_border = cstyle()->border_col_focus;
		}

		painter()->setPen ( Qt::NoPen );
		painter()->setBrush ( col_border );
		spainter.draw_bevel_rect ( img_rect, bevel );
	}


	// Area
	QColor col_dk ( cstyle()->col_bg );
	QColor col_win ( palette().color ( QPalette::Button ) );

	QRadialGradient grad;
	{
		const qreal rad ( _min_len / 2.0 - _border_width );
		const QPointF center ( is().width() / 2.0, is().height() / 2.0 );

		grad.setRadius ( rad );
		grad.setCenter ( center );
		grad.setFocalPoint ( center );

		// Color mixing
		int w_dk ( 40 );
		if ( mouse_over() ) {
			w_dk = 55;
		}

		QColor col_grad_base ( col_win );
		QColor col_grad_dk (
			Wdg::Painter::col_mix ( col_dk, col_win, w_dk, 255 - w_dk ) );

		grad.setColorAt ( 0, col_grad_base );
		grad.setColorAt ( 1, col_grad_dk );
	}

	painter()->setPen ( Qt::NoPen );
	painter()->setBrush ( grad );
	{
		QRect re_area ( img_rect );
		re_area.adjust ( bw, bw, -bw, -bw );
		const qreal bevel2 ( bevel + bw*( sqrt ( 2.0 ) - 2.0 ) );
		spainter.draw_bevel_rect ( re_area, bevel2 );
	}
}


void
QCheck_Box_Painter_Box::paint_bg_frame ( )
{
	QWidget_Style_Painter spainter ( painter() );
	QRect img_rect ( QPoint ( 0, 0 ), is() );

	// Mouse over
	if ( mouse_over() ) {
		QColor col_hl ( palette().color ( QPalette::Highlight ) );
		painter()->setBrush ( col_hl );

		const qreal bw ( _border_width );
		QRectF rect ( QPoint ( 0, 0 ), is() );
		rect.adjust ( bw, bw, -bw, -bw );

		spainter.draw_bevel_frame ( rect, bw, bw );
	}
}


int
QCheck_Box_Painter_Box::paint_handle (
	SPix::Shared_Pixmap_Set * spms_n )
{
	if ( spms_n->pixmap_size().isValid() ) {
		set_is ( spms_n->pixmap_size() );

		paint_handle_type ( spms_n, 0, 0 );
		paint_handle_type ( spms_n, 0, 1 );
		paint_handle_type ( spms_n, 1, 0 );
		paint_handle_type ( spms_n, 1, 1 );

		return 0;
	}
	return -1;
}



void
QCheck_Box_Painter_Box::paint_handle_type (
	SPix::Shared_Pixmap_Set * spms_n,
	int dir_n,
	int checked_n )
{
	QImage img ( is(), QImage::Format_ARGB32 );

	QColor col;
	QColor col_bg;

	QPainter painter ( &img );
	painter.setRenderHints (
		QPainter::Antialiasing |
		QPainter::TextAntialiasing |
		QPainter::SmoothPixmapTransform );


	// Clear image
	img.fill ( 0 );

	// Draw frame
	col = cstyle()->col_fg;
	painter.setPen ( Qt::NoPen );
	painter.setBrush ( col );

	const int lmin ( qMin ( is().width(), is().height() ) );
	const int bw ( 1 ); // Border width
	const float lw ( qMax ( float ( lmin ) / 8.0f, 1.0f ) ); // Line width

	const float center_x ( float ( is().width() ) / 2.0f );
	const float center_y ( float ( is().height() ) / 2.0f );

	const float ww ( float ( lmin - 2*bw ) - 2.0f*lw - 2.0f );

	float pmin_x ( center_x - ww / 2.0 );
	float pmax_x ( center_x + ww / 2.0 );

	float pmin_y ( center_y - ww / 2.0 );
	float pmax_y ( center_y + ww / 2.0 );

	QPointF p_s ( pmin_x, pmin_y );
	QPointF p_e ( pmax_x, pmax_y );
	painter.drawPolygon ( cross_line ( p_s, p_e, lw ) );

	p_s = QPointF ( pmin_x, pmax_y );
	p_e = QPointF ( pmax_x, pmin_y );
	painter.drawPolygon ( cross_line ( p_s, p_e, lw ) );

	spms_n->pixmap_ref ( dir_n*2 + checked_n ) = QPixmap::fromImage ( img );
}


QPolygonF
QCheck_Box_Painter_Box::cross_line (
	QPointF start_n,
	QPointF end_n,
	float width_n )
{
	QPolygonF po;

	const float ds_x ( -width_n );
	const float de_x ( width_n );
	float ds_y ( -width_n );
	float de_y ( width_n );

	if ( start_n.y() > end_n.y() ) {
		ds_y = -ds_y;
		de_y = -de_y;
	}

	po << QPointF (
		start_n.x(),
		start_n.y() + ds_y );

	po << QPointF (
		end_n.x() + de_x,
		end_n.y() );

	po << QPointF (
		end_n.x(),
		end_n.y() + de_y );

	po << QPointF (
		start_n.x() + ds_x,
		start_n.y() );

	po << po[0];

	return po;
}


} // End of namespace
} // End of namespace

