//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "qcovered_input.hpp"

#include <QEvent>
#include <QFocusEvent>


namespace Wdg
{


QCovered_Input::QCovered_Input (
	QWidget * parent_n ) :
QWidget ( parent_n ),
_label ( this ),
_input ( 0 )
{
	_lay_stack.setContentsMargins ( 0, 0, 0, 0 );
	_lay_stack.addWidget ( label() );

	setLayout ( &_lay_stack );
}


void
QCovered_Input::set_input_wdg (
	QWidget * wdg_n )
{
	if ( _input != 0 ) {
		_input->removeEventFilter ( this );
	}

	_input = wdg_n;

	if ( _input != 0 ) {
		_lay_stack.addWidget ( _input );
		_input->installEventFilter ( this );
	}
}


void
QCovered_Input::switch_to_label ( )
{
	_lay_stack.setCurrentIndex ( 0 );
}


void
QCovered_Input::switch_to_input ( )
{
	if ( input() != 0 ) {
		_lay_stack.setCurrentIndex ( 1 );
	}
}


void
QCovered_Input::switch_to_input (
	bool show_n )
{
	if ( show_n ) {
		switch_to_input();
	} else {
		switch_to_label();
	}
}


bool
QCovered_Input::event (
	QEvent * event_n )
{
	if ( isEnabled() ) {
		if ( event_n->type() == QEvent::Enter ) {
			switch_to_input();
		} else if ( event_n->type() == QEvent::Leave ) {
			if ( input() != 0 ) {
				if ( !input()->hasFocus() ) {
					switch_to_label();
				}
			}
		}
	} else {
		if ( event_n->type() == QEvent::EnabledChange ) {
			switch_to_label();
		}
	}

	return QWidget::event ( event_n );
}


bool
QCovered_Input::eventFilter (
	QObject * watched,
	QEvent * event_n )
{
	if ( ( input() != 0 ) && ( watched == input() ) ) {
		if ( event_n->type() == QEvent::FocusOut ) {
			if ( !input()->underMouse() ) {
				switch_to_label();
			}
		}
	}
	return false;
}


} // End of namespace

