//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_qslider_painter_hpp__
#define __INC_qslider_painter_hpp__

#include "qslider_pixmaps_config.hpp"
#include "qwidget_painter.hpp"


namespace Wdg
{
namespace Painter
{


///
/// @brief QSlider_Painter
///
class QSlider_Painter :
	public QWidget_Painter
{
	// Public methods
	public:

	QSlider_Painter ( );

	int
	tick_min_dist ( ) const;

	void
	set_tick_min_dist (
		int delta_n );


	const SPix::Shared_Pixmap_Set_Config *
	pm_cfg ( ) const;

	const QSlider_Pixmaps_Config_Bg *
	pm_cfg_bg ( ) const;


	int
	paint_pixmaps (
		SPix::Shared_Pixmap_Set * spms_n,
		int image_type_n );


	// Protected methods
	protected:

	///
	/// @brief Paints background images
	///
	/// 0 - idle
	/// 1 - focus
	/// 2 - weak focus
	///
	virtual
	int
	paint_bg (
		SPix::Shared_Pixmap_Set * spms_n ) = 0;


	///
	/// @brief Paints handle images
	///
	/// 0 - idle
	/// 1 - focus
	/// 2 - idle + hover
	/// 3 - focus + hover
	/// 4 - isDown()
	/// 5 - Current value marker
	/// 7 - Value hint marker
	///
	virtual
	int
	paint_handle (
		SPix::Shared_Pixmap_Set * spms_n ) = 0;


	bool
	has_focus ( ) const;

	bool
	has_weak_focus ( ) const;

	bool
	mouse_over ( ) const;

	bool
	is_down ( ) const;


	void
	set_has_focus (
		bool flag_n );

	void
	set_has_weak_focus (
		bool flag_n );

	void
	set_mouse_over (
		bool flag_n );

	void
	set_is_down (
		bool flag_n );


	// Private attributes
	private:

	int _tick_min_dist;

	const SPix::Shared_Pixmap_Set_Config * _pm_cfg;

	const QSlider_Pixmaps_Config_Bg * _pm_cfg_bg;


	bool _has_focus;

	bool _has_weak_focus;

	bool _mouse_over;

	bool _is_down;
};


inline
int
QSlider_Painter::tick_min_dist ( ) const
{
	return _tick_min_dist;
}


inline
void
QSlider_Painter::set_tick_min_dist (
	int delta_n )
{
	_tick_min_dist = delta_n;
}


inline
const SPix::Shared_Pixmap_Set_Config *
QSlider_Painter::pm_cfg ( ) const
{
	return _pm_cfg;
}


inline
const QSlider_Pixmaps_Config_Bg *
QSlider_Painter::pm_cfg_bg ( ) const
{
	return _pm_cfg_bg;
}



inline
bool
QSlider_Painter::has_focus ( ) const
{
	return _has_focus;
}


inline
bool
QSlider_Painter::has_weak_focus ( ) const
{
	return _has_weak_focus;
}


inline
bool
QSlider_Painter::mouse_over ( ) const
{
	return _mouse_over;
}


inline
bool
QSlider_Painter::is_down ( ) const
{
	return _is_down;
}


inline
void
QSlider_Painter::set_has_focus (
	bool flag_n )
{
	_has_focus = flag_n;
}


inline
void
QSlider_Painter::set_has_weak_focus (
	bool flag_n )
{
	_has_weak_focus = flag_n;
}


inline
void
QSlider_Painter::set_mouse_over (
	bool flag_n )
{
	_mouse_over = flag_n;
}


inline
void
QSlider_Painter::set_is_down (
	bool is_down_n )
{
	_is_down = is_down_n;
}




} // End of namespace
} // End of namespace


#endif


