//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "qslider_painter_box.hpp"

#include "range_mapper.hpp"
#include <cmath>

#include <iostream>


namespace Wdg
{
namespace Painter
{


QSlider_Painter_Box::QSlider_Painter_Box ( )
{
}


int
QSlider_Painter_Box::paint_bg (
	SPix::Shared_Pixmap_Set * spms_n )
{
	int res ( -1 );
	if ( pm_cfg_bg()->size.isValid() ) {
		set_is ( pm_cfg_bg()->size );

		_bw = is().width() / 16;
		_bw = qMax ( _bw, 2 );

		_ew = qMax ( 1, ( _bw / 4 ) );
		_bevel = _bw;

		calc_ticks();

		paint_bg_base();

		set_has_focus ( false );
		set_has_weak_focus ( false );
		paint_bg_type ( spms_n, 0 );

		set_has_focus ( true );
		set_has_weak_focus ( false );
		paint_bg_type ( spms_n, 1 );

		set_has_focus ( true );
		set_has_weak_focus ( true );
		paint_bg_type ( spms_n, 2 );

		// Clear base image
		_img_base = QImage();
		res = 0;
	}
	return res;
}


void
QSlider_Painter_Box::calc_ticks ( )
{
	int in_width ( is().width() - 2*_bw );
	_tick_width[0] = in_width * 1 / 2;
	_tick_width[1] = in_width * 1 / 4;
	_tick_width[2] = in_width * 1 / 16;

	_tick_width[0] = std::max ( _tick_width[0], 6 );
	_tick_width[1] = std::max ( _tick_width[1], 4 );
	_tick_width[2] = std::max ( _tick_width[2], 2 );

	if ( ( is().width() % 2 ) == 0 ) {
		for ( int ii=0; ii < 3; ++ii ) {
			if ( ( _tick_width[ii] % 2 ) != 0 ) {
				++( _tick_width[ii] );
			}
		}
	} else {
		for ( int ii=0; ii < 3; ++ii ) {
			if ( ( _tick_width[ii] % 2 ) == 0 ) {
				++( _tick_width[ii] );
			}
		}
	}
}


void
QSlider_Painter_Box::paint_bg_base ( )
{
	_img_base = QImage ( is(), QImage::Format_ARGB32 );
	_img_base.fill ( 0 );

	QPainter pnt ( &_img_base );
	pnt.setRenderHints (
		QPainter::Antialiasing |
		QPainter::TextAntialiasing |
		QPainter::SmoothPixmapTransform );

	set_painter ( &pnt );

	paint_bg_area();
	paint_bg_area_deco();
	paint_bg_ticks();
	paint_bg_frame();
}


void
QSlider_Painter_Box::paint_bg_type (
	SPix::Shared_Pixmap_Set * spms_n,
	int pixmap_idx_n )
{
	QImage img ( _img_base );

	{
		QPainter pnt ( &img );
		pnt.setRenderHints (
			QPainter::Antialiasing |
			QPainter::TextAntialiasing |
			QPainter::SmoothPixmapTransform );

		set_painter ( &pnt );

		paint_bg_frame_deco();
	}

	spms_n->pixmap_ref ( pixmap_idx_n ) = QPixmap::fromImage ( img );
}


void
QSlider_Painter_Box::paint_bg_area ( )
{
	QWidget_Style_Painter spainter ( painter() );
	const QRect img_rect ( QPoint ( 0, 0 ), is() );

	QColor col_snd ( cstyle()->col_bg );
	QColor col_snd_trans ( cstyle()->col_bg );
	QColor col_li ( palette().color ( QPalette::Button ) );
	QColor col_mid ( palette().color ( QPalette::Button ) );
	QColor col_dk ( palette().color ( QPalette::Mid ) );

	col_li.setAlpha ( 0 );
	col_dk.setAlpha ( 190 );

	col_snd.setAlpha ( 48 );
	col_snd_trans.setAlpha ( 0 );


	// Inner area
	const QRectF re_in (
		QPoint ( _bw, _bw ),
		QSize ( is().width() - 2*_bw, is().height() - 2*_bw ) );

	const qreal x_start = _bw;
	const qreal x_end = is().width() - _bw;
	const qreal x_mid = x_start + re_in.width() / 4;


	QPainterPath path_area;
	const qreal bev ( _bevel + ( _bw + 1.5*_ew ) * ( sqrt ( 2.0 ) - 2.0 ) );
	spainter.setup_bevel_rect ( path_area, re_in,  bev );

	painter()->setPen ( Qt::NoPen );
	painter()->setBrush ( col_mid );
	painter()->drawPath ( path_area );


	// Sound color
	{
		QLinearGradient gr ( x_start, 0.0, x_mid, 0.0 );
		gr.setColorAt ( 0, col_snd );
		gr.setColorAt ( 1, col_snd_trans );

		painter()->setBrush ( gr );
		painter()->drawPath ( path_area );
	}

	// 3D fake gradient
	{
		QLinearGradient gr ( x_start, 0.0, x_end, 0.0 );
		gr.setColorAt ( 0.0, col_li );
		gr.setColorAt ( x_mid / ( x_end - x_start ), col_mid );
		gr.setColorAt ( 1.0, col_dk );

		painter()->setBrush ( gr );
		painter()->drawPath ( path_area );
	}
}


void
QSlider_Painter_Box::paint_bg_area_deco ( )
{
	const int range_max_idx ( pm_cfg_bg()->ticks_range_max_idx );
	int y_min;
	int y_bottom;
	int y_top;

	y_bottom = is().height() - 1;
	y_bottom -= pm_cfg_bg()->ticks_range_start;
	y_top = y_bottom - range_max_idx;

	if ( pm_cfg_bg()->bg_show_image == 0 ) {
		return;
	} else {
		Wdg::Range_Mapper_Auto mp ( range_max_idx, pm_cfg_bg()->ticks_max_idx );
		y_min = y_bottom - mp.v2_to_v1 ( pm_cfg_bg()->bg_tick_min_idx );
	}

	const bool has_two ( ( y_min > y_top ) && ( y_min < y_bottom ) );
	const qreal w_tick ( ( _tick_width[0] + _tick_width[1] ) / 2.0 );
	const qreal w_out ( w_tick * 2.0 / 3.0 );
	const qreal w_neck ( 1 );

	qreal x_out_l ( ( is().width() - w_tick ) / 2.0 );
	qreal x_neck_l ( x_out_l );
	qreal x_out_r;
	qreal x_neck_r;

	x_neck_l = std::floor ( x_neck_l );
	x_neck_r = x_neck_l + w_neck;

	x_out_l = std::floor ( x_out_l );
	x_out_r = x_out_l + w_out;

	QColor col_pen ( cstyle()->col_fg );
	QColor col_fill ( cstyle()->col_bg );
	col_pen.setAlpha ( 48 );
	col_fill.setAlpha ( 32 );

	painter()->setPen ( col_pen );
	painter()->setBrush ( col_fill );

	if ( has_two ) {
		qreal x_out_rt ( x_out_r );
		qreal x_out_rb ( x_out_r );

		const int dt ( y_min - y_top );
		const int db ( y_bottom - y_min );
		if ( dt > db ) {
			int ww = ( db * w_out ) / dt;
			x_out_rb = x_out_l + ww;
		} else {
			int ww = ( dt * w_out ) / db;
			x_out_rt = x_out_l + ww;
		}

		const unsigned int num_pts = 6;
		QPointF pts[num_pts] = {
			QPointF ( x_out_l,  y_top + 0.5 ),
			QPointF ( x_out_rt, y_top + 0.5 ),
			QPointF ( x_neck_r, y_min + 0.5 ),
			QPointF ( x_out_rb, y_bottom + 0.5 ),
			QPointF ( x_out_l,  y_bottom + 0.5 ),
			QPointF ( x_neck_l, y_min + 0.5 )
		};
		painter()->drawPolygon ( pts, num_pts );

	} else {
		qreal x_rt;
		qreal x_rb;
		if ( y_min < y_bottom ) {
			x_rt = x_neck_r;
			x_rb = x_out_r;
		} else {
			x_rt = x_out_r;
			x_rb = x_neck_r;
		}

		const unsigned int num_pts = 4;
		QPointF pts[num_pts] = {
			QPointF ( x_neck_l, y_top + 0.5 ),
			QPointF ( x_rt,     y_top + 0.5 ),
			QPointF ( x_rb,     y_bottom + 0.5 ),
			QPointF ( x_out_l,  y_bottom + 0.5 )
		};
		painter()->drawPolygon ( pts, num_pts );
	}
}


void
QSlider_Painter_Box::paint_bg_frame ( )
{
	QWidget_Style_Painter spainter ( painter() );

	QRect re_img ( QPoint ( 0, 0 ), is() );

	//spainter.draw_raised_frame (
	//	re_img, _bw[0], _bw[1], _ew, _ew,
	//	palette().color ( QPalette::Button ) );

	spainter.draw_bevel_raised_frame (
		re_img, _bw, _ew, _bevel,
		palette().color ( QPalette::Button ) );
}


void
QSlider_Painter_Box::paint_bg_frame_deco ( )
{
	QWidget_Style_Painter spainter ( painter() );
	QRectF img_rect ( QPoint ( 0, 0 ), is() );

	const QColor col_norm ( palette().color ( QPalette::Button ) );
	const QColor col_focus ( palette().color ( QPalette::WindowText ) );
	const QColor col_weak_focus ( palette().color ( QPalette::Highlight ) );
	QColor bcol ( Wdg::Painter::col_mix ( col_focus, col_norm, 3, 1 ) );

	if ( has_focus() ) {
		if ( has_weak_focus() ) {
			bcol = col_weak_focus;
		}
		painter()->setPen ( Qt::NoPen );
		painter()->setBrush ( bcol );

		QRectF re_in ( img_rect );
		re_in.adjust ( _ew, _ew, -_ew, -_ew );
		const qreal ww ( qMax ( 1, _bw - 2*_ew ) );
		const qreal bev ( _bevel + _ew * ( sqrt ( 2.0 ) - 2.0 ) );
		spainter.draw_bevel_frame (
			re_in, ww, bev );
	}
}


void
QSlider_Painter_Box::paint_bg_ticks ( )
{
	const int range_max_idx ( pm_cfg_bg()->ticks_range_max_idx );
	unsigned int ticks_max_idx;
	int idx_delta ( 1 );
	int y_off;

	y_off = is().height() - 1;
	y_off -= pm_cfg_bg()->ticks_range_start;

	const int y_bottom = y_off;
	const int y_top    = y_off - range_max_idx;

	// Setup main tick position mapper
	ticks_max_idx = pm_cfg_bg()->ticks_max_idx;
	if ( ticks_max_idx > 0 ) {
		if ( ( range_max_idx / ticks_max_idx ) < 2 ) {
			ticks_max_idx = range_max_idx / tick_min_dist();
		}
	}

	Wdg::Range_Mapper_Up mapper ( ticks_max_idx, range_max_idx );

	if ( ticks_max_idx > 0 ) {
		const int tmd ( mapper.min_dist() );
		while ( ( idx_delta * tmd ) < tick_min_dist() ) {
			++idx_delta;
		}
	}


	// Paint
	painter()->setPen ( Qt::NoPen );

	const int idx_start = idx_delta;
	for ( unsigned int ii=idx_start; ii < ticks_max_idx; ii += idx_delta ) {
		int yy = y_bottom - mapper.map ( ii );
		if ( ( yy - y_top ) < tick_min_dist() ) {
			break;
		}
		paint_bg_tick ( yy, _tick_width[1] );
	}

	{
		// Bottom and top ticks
		if ( ticks_max_idx > 0 ) {
			paint_bg_tick ( y_top, _tick_width[0] );
		}
		paint_bg_tick ( y_bottom, _tick_width[0] );
	}
}


void
QSlider_Painter_Box::paint_bg_tick (
	qreal tick_pos_n,
	qreal tick_width_n )
{
	const qreal tick_start = ( is().width() - tick_width_n ) / 2.0;

	QColor col ( cstyle()->tick_col[0] );


	{
		col.setAlpha ( 32 );
		QPolygonF po;
		qreal xl ( tick_start - 0.5 );
		qreal xr ( tick_start + tick_width_n + 0.5 );
		po << QPointF ( xl, tick_pos_n - 1 );
		po << QPointF ( xr, tick_pos_n - 1 );
		po << QPointF ( xr + 1, tick_pos_n );
		po << QPointF ( xr + 1, tick_pos_n + 1 );
		po << QPointF ( xr, tick_pos_n + 2 );
		po << QPointF ( xl, tick_pos_n + 2 );
		po << QPointF ( xl - 1, tick_pos_n + 1 );
		po << QPointF ( xl - 1, tick_pos_n );

		painter()->setBrush ( col );
		painter()->drawPolygon ( po );
	}

	{
		col.setAlpha ( 200 );
		painter()->setBrush ( col );
		QRectF rect ( tick_start, tick_pos_n, tick_width_n, 1.0 );
		painter()->drawRect ( rect );
	}
}


int
QSlider_Painter_Box::paint_handle (
	SPix::Shared_Pixmap_Set * spms_n )
{
	if ( spms_n->pixmap_size().isValid() ) {

		set_is ( spms_n->pixmap_size() );

		_bw = is().width() / 24;
		_bw = qMin ( _bw, is().height() / 8 );
		_bw = qMax ( _bw, 2 );

		_ew = qMax ( 1, ( _bw / 4 ) );

		set_has_focus ( false );
		set_mouse_over ( false );
		set_is_down ( false );
		paint_handle_type ( spms_n, 0 );

		set_has_focus ( true );
		paint_handle_type ( spms_n, 1 );

		set_has_focus ( false );
		set_mouse_over ( true );
		paint_handle_type ( spms_n, 2 );

		set_has_focus ( true );
		paint_handle_type ( spms_n, 3 );

		set_is_down ( true );
		paint_handle_type ( spms_n, 4 );

		paint_marker_current ( spms_n, 5 );
		paint_marker_hint ( spms_n, 6 );
		return 0;
	}
	return 1;
}


void
QSlider_Painter_Box::paint_handle_type (
	SPix::Shared_Pixmap_Set * spms_n,
	int pixmap_idx_n )
{
	QImage img ( is(), QImage::Format_ARGB32 );
	img.fill ( 0 );

	QPainter pnt ( &img );

	pnt.setRenderHints (
		QPainter::Antialiasing |
		QPainter::TextAntialiasing |
		QPainter::SmoothPixmapTransform );

	set_painter ( &pnt );

	paint_handle_area() ;
	paint_handle_frame();
	paint_handle_items();

	spms_n->pixmap_ref ( pixmap_idx_n ) = QPixmap::fromImage ( img );
}


void
QSlider_Painter_Box::paint_handle_area ( )
{
	QWidget_Style_Painter spainter ( painter() );

	const QRectF img_rect ( QPointF ( 0, 0 ), is() );
	const int iw ( is().width() - 2*_bw );
	//const int ih ( is().height() - 2*_bw );

	QPainterPath path_all;
	spainter.setup_bevel_rect ( path_all, img_rect, _bevel );

	painter()->setPen ( Qt::NoPen );

	// Background
	{
		const qreal grw ( iw / 3.0 );
		const qreal grwn ( grw / qreal ( iw ) );
		QColor col_edge ( cstyle()->col_bg );
		QColor col_center ( cstyle()->col_bg );

		col_edge.setAlpha ( 170 );
		col_center.setAlpha ( 140 );

		// Draw gradient
		QLinearGradient gr ( _bw, 0.0, is().width() - _bw, 0.0 );
		gr.setColorAt ( 0.0, col_edge );
		gr.setColorAt ( grwn, col_center );
		gr.setColorAt ( 1.0 - grwn, col_center );
		gr.setColorAt ( 1.0, col_edge );

		painter()->setBrush ( gr );
		painter()->drawPath ( path_all );
	}


	// Highlight
	{
		QColor col_edge ( cstyle()->col_active_glow );
		QColor col_trans ( cstyle()->col_active_glow );
		col_edge.setAlpha ( 128 );
		col_trans.setAlpha ( 30 );

		const QPointF p_top ( 0.0, _bw + iw / 9.0 );
		const QPointF p_bot ( 0.0, is().height() - _bw );

		QLinearGradient gr ( p_top, p_bot );
		gr.setSpread ( QGradient::ReflectSpread );

		if ( is_down() ) {
			gr.setStart ( p_bot );
			gr.setFinalStop ( p_top );
		} else {
			if ( mouse_over() ) {
				col_edge.setAlpha ( 150 );
			}
		}

		gr.setColorAt ( 0, col_edge );
		gr.setColorAt ( 1, col_trans );

		painter()->setBrush ( gr );
		painter()->drawPath ( path_all );
	}
}


void
QSlider_Painter_Box::paint_handle_frame ( )
{
	QWidget_Style_Painter spainter ( painter() );

	QColor col ( Wdg::Painter::col_mix ( cstyle()->col_bg, cstyle()->col_fg, 2, 3 ) );

	const QRectF rect ( QPointF ( 0, 0 ), is() );
	spainter.draw_bevel_raised_frame (
		rect, _bw, _ew, _bevel,
		col );
}


void
QSlider_Painter_Box::paint_handle_items ( )
{
	// Colors

	QColor col_dark ( cstyle()->col_fg );
	QColor col_light ( palette().color ( QPalette::Light ) );
	col_light = Wdg::Painter::col_mix ( cstyle()->col_bg, col_light, 2, 3 );

	// Transforms

	const QTransform trans_hmirror (
		-1.0, 0.0, 0.0,
		 0.0, 1.0, 0.0,
		 is().width(), 0.0, 1.0 );
	const qreal center_v ( is().height() / 2.0 );

	const qreal bb_w ( 1.0 ); // Width of the bright border
	const qreal down_scale ( 4.0 / 3.0 );
	qreal tri_height_base;
	qreal tri_width;

	{
		const qreal iw ( is().width() - 2*_bw );
		const qreal ih ( is().height() - 2*_bw );
		const qreal tri_len_v ( ih / 6.0f );
		const qreal tri_len_h ( iw / 6.0f );
		const qreal tri_len ( qMin ( tri_len_v, tri_len_h ) );
		tri_height_base = tri_len;
		tri_width = tri_len;
	}

	if ( is_down() ) {
		tri_width /= down_scale;
	}
	tri_width = std::floor ( tri_width + 0.5 );



	// Long triangles

	QPolygonF po_lbr;
	QPolygonF po_ldk;

	{
		const qreal hd ( _bevel + _bw * ( sqrt ( 2.0 ) - 1.0 ) );

		qreal dk_w ( tri_width );
		qreal dk_h ( 0.0 );
		qreal br_w ( 0.0 );
		qreal br_h ( is().height() / 2.0 - hd );

		if ( dk_w > bb_w )  {
			const qreal bbw2 ( bb_w*bb_w );
			const qreal w2 ( dk_w*dk_w );
			const qreal h2 ( br_h*br_h );
			qreal cd ( bbw2 - w2 );
			qreal c0 ( -( 2.0*w2*br_h ) / ( cd * 2.0 ) );
			qreal c1 ( ( w2 * ( bbw2 - h2 ) ) / cd );
			qreal cs ( c0*c0 - c1 );

			if ( cs >= 0 ) {
				dk_h = c0 - sqrt ( cs );
				br_w = dk_w * br_h / dk_h;
			}
		}


		const qreal ll ( _bw );

		// Bright background triangle
		if ( br_w > 0.0 ) {
			po_lbr << QPointF ( ll, center_v - br_h );
			po_lbr << QPointF ( ll + br_w, center_v );
			po_lbr << QPointF ( ll, center_v + br_h );
			po_lbr << po_lbr[0];
		}

		// Dark triangle
		if ( dk_h > 0.0 ) {
			po_ldk << QPointF ( ll, center_v - dk_h );
			po_ldk << QPointF ( ll + dk_w , center_v );
			po_ldk << QPointF ( ll, center_v + dk_h );
			po_ldk << po_ldk[0];
		}
	}

	QPolygonF po_lbr_mirr ( trans_hmirror.map ( po_lbr ) );
	QPolygonF po_ldk_mirr ( trans_hmirror.map ( po_ldk ) );


	// Small triangles

	QPolygonF po_sbr;
	QPolygonF po_sdk;

	{
		qreal tri_height ( tri_height_base );
		tri_height = std::floor ( tri_height ) + 0.5;

		const qreal dk_w ( tri_width );
		const qreal dk_h ( tri_height );
		qreal br_w ( dk_w );
		qreal br_h ( dk_h );
		{
			const qreal dk_r ( ( dk_w*dk_h ) /
				sqrt ( dk_w*dk_w + dk_h*dk_h ) );
			const qreal br_s ( ( dk_r + bb_w ) / dk_r );
			br_w *= br_s;
			br_h *= br_s;
		}

		const qreal ll ( _bw );

		// Bright background triangle
		po_sbr << QPointF ( ll , center_v - br_h );
		po_sbr << QPointF ( ll + br_w, center_v );
		po_sbr << QPointF ( ll , center_v + br_h );
		po_sbr << po_sbr[0];

		// Dark triangle
		po_sdk << QPointF ( ll, center_v - dk_h );
		po_sdk << QPointF ( ll + dk_w, center_v );
		po_sdk << QPointF ( ll, center_v + dk_h );
		po_sdk << po_sdk[0];
	}

	QPolygonF po_sbr_mirr ( trans_hmirror.map ( po_sbr ) );
	QPolygonF po_sdk_mirr ( trans_hmirror.map ( po_sdk ) );


	// Paint
	painter()->setPen ( Qt::NoPen );

	// Paint long triangle
	{
		painter()->setBrush ( col_light );

		painter()->drawPolygon ( po_lbr );
		painter()->drawPolygon ( po_lbr_mirr );

		// Center line
		const QRectF re_line (
			QPointF ( _bw, center_v - 0.5 - 1.0 ),
			QSizeF ( is().width() - 2*_bw, 3.0  ) );
		painter()->drawRect ( re_line );

		painter()->setBrush ( col_dark );

		painter()->drawPolygon ( po_ldk );
		painter()->drawPolygon ( po_ldk_mirr );


	}

	// Paint small triangle
	{
		painter()->setBrush ( col_light );

		painter()->drawPolygon ( po_sbr );
		painter()->drawPolygon ( po_sbr_mirr );

		painter()->setBrush ( col_dark );

		painter()->drawPolygon ( po_sdk );
		painter()->drawPolygon ( po_sdk_mirr );

		const QRectF re_line (
			QPointF ( _bw, center_v - 0.5f ),
			QSizeF ( is().width() - 2*_bw, 1.0 ) );
		painter()->drawRect ( re_line );
	}
}


void
QSlider_Painter_Box::paint_marker_current (
	SPix::Shared_Pixmap_Set * spms_n,
	int img_idx_n )
{
	const int dx ( spms_n->pixmap_width() % 2 );
	const QSize ps ( 6 - dx, 5 );

	QImage img ( ps, QImage::Format_ARGB32 );
	img.fill ( 0 );

	QPainter painter ( &img );
	painter.setRenderHint (	QPainter::Antialiasing );
	painter.setPen ( Qt::NoPen );

	{ // Background
		QColor col ( cstyle()->tick_col[0] );
		col.setAlpha ( 128 );

		const qreal x0 ( 0.0 );
		const qreal x1 ( 2.0 );
		const qreal x2 ( 4.0 - dx );

		const int num_pts ( 9 );
		QPointF pts[num_pts] = {
			QPointF ( x0, 2.0 ),
			QPointF ( x1, x0 ),
			QPointF ( x2, x0 ),
			QPointF ( x2 + 2.0, 2.0 ),
			QPointF ( x2 + 2.0, 3.0 ),
			QPointF ( x2, 5.0 ),
			QPointF ( x1, 5.0 ),
			QPointF ( x0, 3.0 ),
			QPointF ( x0, 2.0 ),
		};

		painter.setBrush ( col );
		painter.drawPolygon ( pts, num_pts );
	}


	{ // Foreground
		QColor col ( cstyle()->tick_col[0] );

		const qreal x0 ( 1.0 );
		const qreal x1 ( 2.0 );
		const qreal x2 ( 4.0 - dx );

		const int num_pts ( 9 );
		QPointF pts[num_pts] = {
			QPointF ( x0, 2.0 ),
			QPointF ( x1, 1.0 ),
			QPointF ( x2, 1.0 ),
			QPointF ( x2 + 1.0, 2.0 ),
			QPointF ( x2 + 1.0, 3.0 ),
			QPointF ( x2, 4.0 ),
			QPointF ( x1, 4.0 ),
			QPointF ( x0, 3.0 ),
			QPointF ( x0, 2.0 )
		};

		painter.setBrush ( col );
		painter.drawPolygon ( pts, num_pts );
	}

	spms_n->pixmap_ref ( img_idx_n ) = QPixmap::fromImage ( img );
}


void
QSlider_Painter_Box::paint_marker_hint (
	SPix::Shared_Pixmap_Set * spms_n,
	int img_idx_n )
{
	const int dx ( spms_n->pixmap_width() % 2 );
	const QSize ps ( 6 - dx, 5 );

	QImage img ( ps, QImage::Format_ARGB32 );
	img.fill ( 0 );

	QPainter painter ( &img );
	painter.setRenderHint (	QPainter::Antialiasing );
	painter.setPen ( Qt::NoPen );

	{
		QColor col ( cstyle()->tick_col[0] );
		col.setAlpha ( 100 );

		const qreal x0 ( 0.0 );
		const qreal x1 ( 2.0 );
		const qreal x2 ( 4.0 - dx );

		const int num_pts ( 9 );
		QPointF pts[num_pts] = {
			QPointF ( x0, 2.0 ),
			QPointF ( x1, x0 ),
			QPointF ( x2, x0 ),
			QPointF ( x2 + 2.0, 2.0 ),
			QPointF ( x2 + 2.0, 3.0 ),
			QPointF ( x2, 5.0 ),
			QPointF ( x1, 5.0 ),
			QPointF ( x0, 3.0 ),
			QPointF ( x0, 2.0 ),
		};

		painter.setBrush ( col );
		painter.drawPolygon ( pts, num_pts );
	}

	spms_n->pixmap_ref ( img_idx_n ) = QPixmap::fromImage ( img );
}


} // End of namespace
} // End of namespace

