//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "qspinbox_float.hpp"

#include <QLineEdit>


namespace Wdg
{


QSpinBox_Float::QSpinBox_Float (
	QWidget * parent ) :
QSpinBox ( parent ),
_decimal_offset ( 0 ),
_decimal_scale ( 1 )
{
}


void
QSpinBox_Float::set_decimal_offset (
	int offset_n )
{
	_decimal_offset = offset_n;
	_decimal_scale = 1;
	for ( int ii=0; ii < _decimal_offset; ++ii ) {
		_decimal_scale *= 10;
	}
}


QString
QSpinBox_Float::textFromValue (
	int value ) const
{
	//return QSpinBox::textFromValue ( value );

	const long gz ( value / _decimal_scale );
	const long lz ( abs ( value ) % _decimal_scale );
	QString res = QString ( "%1" ).arg ( gz );
	res += locale().decimalPoint();
	res += QString ( "%1" ).arg ( lz, 2, 10, QChar ( '0' ) );

	//std::cout << "textFromValue \"";
	//std::cout << res.toLocal8Bit().constData();
	//std::cout << "\"\n";

	return res;
}


int
QSpinBox_Float::valueFromText (
	const QString & text ) const
{
	int res ( 0 );
	QStringList lst ( text.split ( locale().decimalPoint(), QString::SkipEmptyParts ) );
	if ( lst.size() > 0 ) {
		res = lst[0].toInt() * _decimal_scale;
		if ( lst.size() > 1 ) {
			res += lst[1].toInt();
		}
	}

	//std::cout << "valueFromText " << res << "\n";
	return res;
}


QValidator::State
QSpinBox_Float::validate (
	QString & input, int &  ) const
{
	//return QSpinBox::validate ( input, pos );
	QValidator::State state ( QValidator::Invalid );

	int val;
	bool chk;

	QStringList lst ( input.split ( locale().decimalPoint() ) );
	if ( lst.size() > 0 ) {
		lst[0].toInt ( &chk );
		if ( chk ) {
			if ( lst.size() > 1 ) {
				if ( ( _decimal_offset > 0 ) && ( lst.size() == 2 ) ) {
					val = lst[1].toInt ( &chk );
					if ( ( val >= 0 ) && chk ) {
						state = QValidator::Acceptable;
					}
				}
			} else {
				state = QValidator::Acceptable;
			}
		}
	}

	//std::cout << "validate " << state << "\n";
	return state;
}


void
QSpinBox_Float::fixup (
	QString & ) const
{
	return;
}


} // End of namespace

