//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_qwidget_painter_hpp__
#define __INC_qwidget_painter_hpp__

#include "shared_pixmap_set_painter.hpp"
#include "qwidget_style.hpp"

#include <QPalette>
#include <QPainter>
#include <QList>


namespace Wdg
{
namespace Painter
{


class QWidget_Painter :
	public SPix::Shared_Pixmap_Set_Painter
{
	// Public methods
	public:

	QWidget_Painter ( );

	~QWidget_Painter ( );

	const QWidget_Style *
	style (
		int index_n ) const;

	void
	set_style (
		int index_n,
		const QWidget_Style * style_n );


	// Current style

	void
	set_current_style (
		int index_n );

	int
	current_style ( ) const;

	const QWidget_Style *
	cstyle ( ) const;


	// Image size

	const QSize &
	is ( );

	void
	set_is (
		const QSize & size_n );


	// Palette

	QPalette
	palette ( ) const;


	// Painter

	QPainter *
	painter ( ) const;

	void
	set_painter (
		QPainter * painter_n );


	// Private attributes
	private:

	QPainter * _painter;

	QList < const QWidget_Style * >  _styles;

	int _current_style;

	QSize _is; // Image size convenience variable
};


inline
QPainter *
QWidget_Painter::painter ( ) const
{
	return _painter;
}


inline
void
QWidget_Painter::set_painter (
	QPainter * painter_n )
{
	_painter = painter_n;
}


inline
const QWidget_Style *
QWidget_Painter::style (
	int index_n ) const
{
	if ( index_n < _styles.size() ) {
		return _styles[index_n];
	}
	return 0;
}


inline
int
QWidget_Painter::current_style ( ) const
{
	return _current_style;
}


inline
const QWidget_Style *
QWidget_Painter::cstyle ( ) const
{
	return style ( current_style() );
}


inline
const QSize &
QWidget_Painter::is ( )
{
	return _is;
}


inline
void
QWidget_Painter::set_is (
	const QSize & size_n )
{
	_is = size_n;
}


} // End of namespace
} // End of namespace


#endif
