//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "range_mapper.hpp"
#include <iostream>


namespace Wdg
{


Range_Mapper_Down::Range_Mapper_Down (
	ulong max_idx_fine,
	ulong max_idx_coarse ) :
_max_idx_fine ( max_idx_fine ),
_max_idx_coarse ( max_idx_coarse ),
_dist_small ( 0 ),
_dist_large ( 0 ),
_len_mod ( 0 )
{

	if ( _max_idx_coarse > 0 ) {
		_dist_small = _max_idx_fine / _max_idx_coarse;
		_dist_large = _dist_small;
		_len_mod = _max_idx_fine % _max_idx_coarse;

		if ( _len_mod > 0 ) {
			++_dist_large;
		}
	}

	_mid_pos = _len_mod * _dist_large;
}


Range_Mapper::ulong
Range_Mapper_Down::map (
	ulong val_n ) const
{
	ulong res ( 0 );

	if ( _dist_small > 0 ) {
		ulong res_mod;
		ulong mod_lim;

		if ( val_n <= _mid_pos  ) {
			res = val_n / _dist_large;
			res_mod = val_n % _dist_large;
			mod_lim = _dist_large / 2;
		} else {
			val_n -= _mid_pos;
			res = _len_mod;
			res += val_n / _dist_small;
			res_mod = val_n % _dist_small ;
			mod_lim = _dist_small / 2;
		}

		if ( res_mod > mod_lim ) {
			++res;
		}
	}

	return res;
}


Range_Mapper_Up::Range_Mapper_Up (
	ulong max_idx_coarse,
	ulong max_idx_fine ) :
_max_idx_coarse ( max_idx_coarse ),
_max_idx_fine ( max_idx_fine ),
_len_div ( 0 ),
_len_mod ( 0 )
{
	if ( max_idx_coarse > 0 ) {
		_len_div = _max_idx_fine / _max_idx_coarse;
		_len_mod = _max_idx_fine % _max_idx_coarse;
	}
}


Range_Mapper::ulong
Range_Mapper_Up::map (
	ulong val_n ) const
{
	ulong res ( val_n * _len_div );

	if ( val_n >= _len_mod ) {
		res += _len_mod;
	} else {
		res += val_n;
	}

	return res;
}


//
// Range_Mapper_Auto
//
Range_Mapper_Auto::Range_Mapper_Auto (
	ulong max_idx_1_n,
	ulong max_idx_2_n ) :
_max_idx_1 ( 0 ),
_max_idx_2 ( 0 )
{
	set_max_idx_all ( max_idx_1_n, max_idx_2_n );
}


void
Range_Mapper_Auto::set_max_idx_all (
	ulong max_idx_1_n,
	ulong max_idx_2_n )
{
	_max_idx_1 = max_idx_1_n;
	_max_idx_2 = max_idx_2_n;

	ulong lfine;
	ulong lcoarse;

	if ( max_idx_1() > max_idx_2() ) {
		lfine = max_idx_1();
		lcoarse = max_idx_2();
	} else {
		lfine = max_idx_2();
		lcoarse = max_idx_1();
	}

	_mapper_down = Range_Mapper_Down ( lfine, lcoarse );
	_mapper_up = Range_Mapper_Up ( lcoarse, lfine );
}


Range_Mapper::ulong
Range_Mapper_Auto::v1_to_v2 (
	ulong val_n ) const
{
	ulong res;

	if ( max_idx_1() > max_idx_2() ) {
		res = _mapper_down.map ( val_n );
	} else {
		res = _mapper_up.map ( val_n );
	}

	return res;
}


Range_Mapper::ulong
Range_Mapper_Auto::v2_to_v1 (
	ulong val_n ) const
{
	ulong res;

	if ( max_idx_1() > max_idx_2() ) {
		res = _mapper_up.map ( val_n );
	} else {
		res = _mapper_down.map ( val_n );
	}

	return res;
}


} // End of namespace
