//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_wdg_range_mapper_hpp__
#define __INC_wdg_range_mapper_hpp__


namespace Wdg {


class Range_Mapper
{
	public:

	typedef unsigned long int ulong;


	virtual
	ulong
	map (
		ulong val_n ) const = 0;

};


///
/// @brief Maps down to a coarser range
///
class Range_Mapper_Down :
	public Range_Mapper
{
	public:

	///
	/// @param max_idx_fine The maximum index of the fine ladder (num_steps - 1).
	/// @param max_idx_coarse The maximum index of the coarse ladder (num_steps - 1).
	Range_Mapper_Down (
		ulong max_idx_fine = 1,
		ulong max_idx_coarse = 1 );

	ulong
	map (
		ulong val_n ) const;


	// Private attributes
	private:

	ulong _max_idx_fine;
	ulong _max_idx_coarse;

	ulong _dist_small;
	ulong _dist_large;
	ulong _len_mod;
	ulong _mid_pos;
};


///
/// @brief Maps up to a finer range
///
class Range_Mapper_Up :
	public Range_Mapper
{
	public:

	///
	/// @param max_idx_coarse The maximum index of the coarse ladder (num_steps - 1).
	/// @param max_idx_fine The maximum index of the fine ladder (num_steps - 1).
	Range_Mapper_Up (
		ulong max_idx_coarse = 1,
		ulong max_idx_fine = 1 );

	ulong
	map (
		ulong val_n ) const;

	ulong
	min_dist ( ) const;


	// Private attributes
	private:

	ulong _max_idx_coarse;
	ulong _max_idx_fine;

	ulong _len_div;
	ulong _len_mod;
};


inline
Range_Mapper::ulong
Range_Mapper_Up::min_dist ( ) const
{
	return _len_div;
}


class Range_Mapper_Auto
{
	public:

	typedef Range_Mapper::ulong ulong;

	public:

	Range_Mapper_Auto (
		ulong max_idx_1_n = 0,
		ulong max_idx_2_n = 0 );

	ulong
	max_idx_1 ( ) const;

	ulong
	max_idx_2 ( ) const;

	void
	set_max_idx_all (
		ulong max_idx_1_n,
		ulong max_idx_2_n );

	ulong
	v1_to_v2 (
		ulong val_n ) const;

	ulong
	v2_to_v1 (
		ulong val_n ) const;


	// Protected methods
	protected:


	// Private attributes
	private:

	ulong _max_idx_1;
	ulong _max_idx_2;

	Range_Mapper_Up _mapper_up;
	Range_Mapper_Down _mapper_down;
};


inline
Range_Mapper_Auto::ulong
Range_Mapper_Auto::max_idx_1 ( ) const
{
	return _max_idx_1;
}


inline
Range_Mapper_Auto::ulong
Range_Mapper_Auto::max_idx_2 ( ) const
{
	return _max_idx_2;
}


} // End of namespace

#endif


