//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "shared_pixmap_set.hpp"

#include <iostream>


namespace SPix
{


//
// Shared_Pixmap_Set
//

Shared_Pixmap_Set::Shared_Pixmap_Set (
	Shared_Pixmap_Set_Config * cfg_n,
	int num_pixmaps_n ) :
_num_pixmaps ( 0 ),
_pixmaps ( 0 ),
_config ( 0 ),
_num_users ( 0 )
{
	set_config ( cfg_n );
	set_num_pixmaps ( num_pixmaps_n );
}


Shared_Pixmap_Set::Shared_Pixmap_Set (
	QSize size_n,
	int num_pixmaps_n ) :
_num_pixmaps ( 0 ),
_pixmaps ( 0 ),
_config ( 0 ),
_num_users ( 0 )
{
	set_config ( new Shared_Pixmap_Set_Config ( size_n ) );
	set_num_pixmaps ( num_pixmaps_n );
}


Shared_Pixmap_Set::~Shared_Pixmap_Set ( )
{
	clear_pixmaps();
	delete _config;
}


void
Shared_Pixmap_Set::clear_pixmaps ( )
{
	if ( _pixmaps != 0 ) {
		delete[] _pixmaps;
		_pixmaps = 0;
	}
	_num_pixmaps = 0;
}


void
Shared_Pixmap_Set::set_num_pixmaps (
	unsigned int num_n )
{
	if ( num_n != num_pixmaps() ) {
		clear_pixmaps();
		if ( num_n > 0 ) {
			_pixmaps = new QPixmap[num_n];
			_num_pixmaps = num_n;

			if ( pixmap_size().isValid() ) {
				QPixmap pmap ( pixmap_size() );
				set_all_pixmaps ( pmap );
			}
		}
	}
}


void
Shared_Pixmap_Set::set_config (
	Shared_Pixmap_Set_Config * cfg_n )
{
	if ( _config != cfg_n ) {
		if ( _config != 0 ) {
			delete _config;
		}
		_config = cfg_n;
	}

	if ( _config == 0 ) {
		_config = new Shared_Pixmap_Set_Config;
	}

	update_pixmap_sizes();
}


void
Shared_Pixmap_Set::update_pixmap_sizes ( )
{
	if ( num_pixmaps() > 0 ) {
		QPixmap pmap;
		if ( pixmap_size().isValid() ) {
			pmap = QPixmap ( pixmap_size() );
		}
		set_all_pixmaps ( pmap );
	}
}


void
Shared_Pixmap_Set::set_all_pixmaps (
	QPixmap & pman_n )
{
	for ( unsigned int ii=0; ii < num_pixmaps(); ++ii ) {
		_pixmaps[ii] = pman_n;
	}
}


} // End of namespace

