//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_wdg_shared_pixmap_set_hpp__
#define __INC_wdg_shared_pixmap_set_hpp__

#include <QPixmap>
#include "shared_pixmap_set_config.hpp"


namespace SPix
{


///
/// @brief Shared_Pixmap_Set
///
class Shared_Pixmap_Set
{
	// Public methods
	public:

	Shared_Pixmap_Set (
		Shared_Pixmap_Set_Config * cfg_n = 0,
		int num_pm_n = 0 );

	Shared_Pixmap_Set (
		QSize size_n,
		int num_pm_n = 0 );

	virtual
	~Shared_Pixmap_Set ( );


	void
	clear_pixmaps();


	// Image size

	const QSize &
	pixmap_size ( ) const;

	int
	pixmap_width ( ) const;

	int
	pixmap_height ( ) const;


	// Number of pixmaps

	unsigned int
	num_pixmaps ( ) const;

	void
	set_num_pixmaps (
		unsigned int num_n );


	// Pixmap buffer

	const QPixmap &
	pixmap (
		int index_n ) const;

	QPixmap &
	pixmap_ref (
		int index_n );


	// Configuration

	const Shared_Pixmap_Set_Config *
	config ( ) const;

	void
	set_config (
		Shared_Pixmap_Set_Config * cfg_n );


	// Users

	unsigned int
	num_users ( ) const;

	void
	increment_num_users ( );

	void
	decrement_num_users ( );


	// Private methods
	private:

	void
	update_pixmap_sizes ( );

	void
	set_all_pixmaps (
		QPixmap & pman_n );


	// Private attributes
	private:

	unsigned int _num_pixmaps;

	QPixmap * _pixmaps;

	Shared_Pixmap_Set_Config * _config;


	unsigned int _num_users;
};



inline
const QSize &
Shared_Pixmap_Set::pixmap_size ( ) const
{
	return config()->size;
}


inline
int
Shared_Pixmap_Set::pixmap_width ( ) const
{
	return pixmap_size().width();
}


inline
int
Shared_Pixmap_Set::pixmap_height ( ) const
{
	return pixmap_size().height();
}


inline
unsigned int
Shared_Pixmap_Set::num_pixmaps ( ) const
{
	return _num_pixmaps;
}


inline
const QPixmap &
Shared_Pixmap_Set::pixmap (
	int index_n ) const
{
	return _pixmaps[index_n];
}


inline
QPixmap &
Shared_Pixmap_Set::pixmap_ref (
	int index_n )
{
	return _pixmaps[index_n];
}


inline
const Shared_Pixmap_Set_Config *
Shared_Pixmap_Set::config ( ) const
{
	return _config;
}


inline
unsigned int
Shared_Pixmap_Set::num_users ( ) const
{
	return _num_users;
}


inline
void
Shared_Pixmap_Set::increment_num_users ( )
{
	++_num_users;
}


inline
void
Shared_Pixmap_Set::decrement_num_users ( )
{
	--_num_users;
}


} // End of namespace


#endif


