//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "shared_pixmap_set_buffer.hpp"

#include <iostream>
#include <algorithm>
#include <cassert>


namespace SPix
{


//
// Shared_Pixmap_Set_Buffer
//
Shared_Pixmap_Set_Buffer::Shared_Pixmap_Set_Buffer (
	int pixmap_type_n ) :
_pixmap_type ( pixmap_type_n )
{
}


Shared_Pixmap_Set_Buffer::~Shared_Pixmap_Set_Buffer ( )
{
	clear();
}


void
Shared_Pixmap_Set_Buffer::clear ( )
{
	clear_sets();
	set_painter ( 0 );
}


void
Shared_Pixmap_Set_Buffer::clear_sets ( )
{
	if ( _list.size() > 0 ) {
		for ( int ii=0; ii < _list.size(); ++ii ) {
			delete _list[ii];
		}
		_list.clear();
	}
}


void
Shared_Pixmap_Set_Buffer::set_pixmap_type (
	int type_n )
{
	_pixmap_type = type_n;
}


void
Shared_Pixmap_Set_Buffer::set_painter (
	Shared_Pixmap_Set_Painter * painter_n )
{
	_painter = painter_n;
}


Shared_Pixmap_Set *
Shared_Pixmap_Set_Buffer::acquire (
	const Shared_Pixmap_Set_Config & cfg_n )
{
	Shared_Pixmap_Set * res ( 0 );

	if ( !cfg_n.size.isValid() ) {
		return res;
	}

	// Try to find and pixmap set witch a matching config
	for ( int ii=0; ii < _list.size(); ++ii ) {
		Shared_Pixmap_Set * sps ( _list[ii] );

		if ( cfg_n.equal ( *sps->config() ) ) {
			sps->increment_num_users();
			res = sps;
			break;
		}
	}

	if ( res == 0 ) {
		if ( painter() != 0 ) {
			Shared_Pixmap_Set * pmap (
				new Shared_Pixmap_Set ( cfg_n.new_clone() ) );

			int err = painter()->paint_pixmaps ( pmap, pixmap_type() );
			if ( err < 0 ) {
				std::cerr << "EE: Shared_Pixmap_Set_Buffer: Painting failed\n";
				delete pmap;
			} else {
				pmap->increment_num_users();
				_list.append ( pmap );
				res = pmap;
			}

		} else {
			std::cerr << "EE: Shared_Pixmap_Set_Buffer: No painter set\n";
		}
	}

	return res;
}


Shared_Pixmap_Set *
Shared_Pixmap_Set_Buffer::return_acquire (
	Shared_Pixmap_Set * ret_pxmaps_n,
	const Shared_Pixmap_Set_Config & cfg_n )
{
	Shared_Pixmap_Set * res ( acquire ( cfg_n ) );
	return_pixmaps ( ret_pxmaps_n );
	return res;
}


void
Shared_Pixmap_Set_Buffer::return_pixmaps (
	Shared_Pixmap_Set * pxmaps_n )
{
	if ( pxmaps_n != 0 ) {
		for ( int ii=0; ii < _list.size(); ++ii ) {
			Shared_Pixmap_Set * sps ( _list[ii] );
			if ( sps == pxmaps_n ) {
				sps->decrement_num_users();
				if ( sps->num_users() <= 0 ) {
					delete sps;
					_list.removeAt ( ii );
				}
				break;
			}
		}
	}

	//std::cout << "List size " << _list.size() << "\n";
}



} // End of namespace

