//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "side_interface.hpp"

#include <iostream>


Side_Interface::Side_Interface (
	QWidget * parent ) :
QFrame ( parent )
{
	_mg = 6;

	// Control selection

	_controls_view.setHorizontalScrollBarPolicy ( Qt::ScrollBarAlwaysOff );
	_controls_view.setFrameStyle ( QFrame::StyledPanel | QFrame::Plain );

	connect ( &_controls_view, SIGNAL ( activated ( const QModelIndex & ) ),
		this, SLOT ( control_selected ( const QModelIndex & ) ) );

	connect ( &_controls_view, SIGNAL ( sig_layout_changed() ),
		this, SLOT ( update_layout() ) );

	_controls_view.setModel ( &_controls_model );
	_controls_view.setExpanded ( _controls_model.item_cards()->index(), true );
	_controls_view.setExpanded ( _controls_model.item_plugs()->index(), true );


	// Direction

	QLabel * lbl_dir = new QLabel ( "<h3>" + tr ( "Show elements" ) + "</h3>" );

	_snd_dir_check[0].setText ( tr ( "Playback" ) );
	_snd_dir_check[0].setToolTip ( tr ( "Show playback elements" ) );
	_snd_dir_check[1].setText ( tr ( "Capture" ) );
	_snd_dir_check[1].setToolTip ( tr ( "Show capture elements" ) );

	connect ( &_snd_dir_check[0] , SIGNAL ( toggled ( bool ) ),
		this, SLOT ( dir_check_toggled ( bool ) ) );
	connect ( &_snd_dir_check[1] , SIGNAL ( toggled ( bool ) ),
		this, SLOT ( dir_check_toggled ( bool ) ) );


	// Compose layout
	QVBoxLayout * lay_cards = new QVBoxLayout();
	lay_cards->setContentsMargins ( _mg, _mg, _mg, _mg );
	//lay_cards->addWidget ( lbl_cards, 0 );
	lay_cards->addWidget ( &_controls_view, 1 );

	QWidget * wdg_cards = new QWidget();
	wdg_cards->setLayout ( lay_cards );

	QVBoxLayout * lay_dir = new QVBoxLayout();
	lay_dir->setContentsMargins ( _mg, _mg, _mg, _mg );
	lay_dir->addWidget ( lbl_dir );
	lay_dir->addWidget ( &_snd_dir_check[0] );
	lay_dir->addWidget ( &_snd_dir_check[1] );
	lay_dir->addStretch ( 1 );

	QWidget * wdg_dir = new QWidget();
	wdg_dir->setLayout ( lay_dir );


	// Splitter
	_splitter.setOrientation ( Qt::Vertical );
	_splitter.addWidget ( wdg_cards );
	_splitter.addWidget ( wdg_dir );
	_splitter.setCollapsible ( 0, false );
	_splitter.setCollapsible ( 1, false );
	_splitter.setStretchFactor ( 0, 0 );
	_splitter.setStretchFactor ( 1, 1 );

	QVBoxLayout * lay_vbox ( new QVBoxLayout() );
	lay_vbox->setContentsMargins ( 0, 0, 0, 0 );
	lay_vbox->addWidget ( &_splitter );
	setLayout ( lay_vbox );

	update_layout();
}


Side_Interface::~Side_Interface ( )
{
}


void
Side_Interface::set_mixer_config (
	const QSnd::Mixer_Config & cfg_n )
{
	if ( _mixer_config != cfg_n ) {
		_mixer_config = cfg_n;

		for ( int ii=0; ii < 2; ++ii ) {
			QCheckBox & cbox ( _snd_dir_check[ii] );
			cbox.blockSignals ( true );
			cbox.setChecked ( _mixer_config.show_dir[ii] );
			cbox.blockSignals ( false );
		}

		emit sig_mixer_config_changed();
	}
}


void
Side_Interface::set_current_ctl (
	const QSnd::Snd_Control_Info & ctl_n )
{
	if ( _current_ctl != ctl_n ) {
		QModelIndex midx (
			_controls_model.ctl_info_index ( ctl_n ) );

		_controls_view.blockSignals ( true );
		_controls_view.setCurrentIndex ( midx );
		_controls_view.blockSignals ( false );

		if ( midx.isValid() ) {
			_current_ctl = *_controls_model.ctl_info ( midx );
		} else {
			_current_ctl.clear();
		}
	}
}


void
Side_Interface::control_selected (
	const QModelIndex & idx_n )
{
	const QSnd::Snd_Control_Info * ctl_info ( 0 );

	//std::cout << "Side_Interface::control_selected " << "\n";

	if ( idx_n.isValid() && ( _controls_view.model() != 0 ) ) {
		ctl_info = _controls_model.ctl_info ( idx_n );
	}

	if ( ctl_info != 0 ) {
		if ( _current_ctl != (*ctl_info) ) {
			_current_ctl = *ctl_info;
			if ( !( _snd_dir_check[0].isChecked() ||
			        _snd_dir_check[1].isChecked() ) )
			{
				_snd_dir_check[0].setChecked ( true );
			}
			emit sig_control_changed();
		}
	}

}


void
Side_Interface::dir_check_toggled (
	bool )
{
	bool changed ( false );

	for ( int ii=0; ii < 2; ++ii ) {
		if ( _snd_dir_check[ii].isChecked() != _mixer_config.show_dir[ii] ) {
			_mixer_config.show_dir[ii] = _snd_dir_check[ii].isChecked();
			changed = true;
		}
	}

	if ( changed ) {
		emit sig_mixer_config_changed();
	}
}


void
Side_Interface::update_layout ( )
{
	update_maximum_size();
}


void
Side_Interface::refresh_data ( )
{
	_controls_model.update_data();

	QModelIndex midx (
		_controls_model.ctl_info_index ( _current_ctl ) );
	_controls_view.setCurrentIndex ( midx );
}


void
Side_Interface::update_maximum_size ( )
{
	int max_ww = _controls_view.maximumWidth();
	max_ww += 2*( frameWidth() + _mg );
	max_ww = qMax ( sizeHint().width(), max_ww );

	if ( maximumWidth() != max_ww ) {
		setMaximumWidth ( max_ww );
	}
	updateGeometry();
}

