//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_qside_interface_hpp__
#define __INC_qside_interface_hpp__

#include <QFrame>
#include <QLabel>
#include <QPushButton>
#include <QCheckBox>
#include <QSplitter>
#include <QVBoxLayout>

#include "snd_controls_model.hpp"
#include "tree_view_controls.hpp"
#include "mixer_config.hpp"


class Side_Interface :
	public QFrame
{
	Q_OBJECT

	public:

	Side_Interface (
		QWidget * parent = 0 );

	~Side_Interface ( );


	const QSnd::Snd_Control_Info &
	default_ctl ( ) const;

	const QSnd::Snd_Control_Info &
	current_ctl ( ) const;

	void
	set_current_ctl (
		const QSnd::Snd_Control_Info & ctl_n );


	// Mixer configuration

	const QSnd::Mixer_Config &
	mixer_config ( ) const;

	void
	set_mixer_config (
		const QSnd::Mixer_Config & cfg_n );


	// Signals
	signals:

	void
	sig_control_changed ( );

	void
	sig_mixer_config_changed ( );

	void
	sig_snd_dir_changed (
		int idx_n );


	// Public slots
	public slots:

	void
	update_layout ( );

	void
	refresh_data ( );


	// Protected slots
	protected slots:

	void
	control_selected (
		const QModelIndex & idx_n );

	void
	dir_check_toggled (
		bool checked_n );

	void
	update_maximum_size ( );


	// Private attributes
	private:

	QSnd::Snd_Controls_Model _controls_model;

	QSnd::Snd_Control_Info _current_ctl;

	QSnd::Mixer_Config _mixer_config;


	// Widgets

	QSplitter _splitter;

	QSnd::Tree_View_Controls _controls_view;

	QCheckBox _snd_dir_check[2];

	int _mg;
};


inline
const QSnd::Snd_Control_Info &
Side_Interface::default_ctl ( ) const
{
	return _controls_model.default_ctl();
}


inline
const QSnd::Snd_Control_Info &
Side_Interface::current_ctl ( ) const
{
	return _current_ctl;
}


inline
const QSnd::Mixer_Config &
Side_Interface::mixer_config ( ) const
{
	return _mixer_config;
}


#endif


