//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "sliders_pad.hpp"

#include <QPainter>

#include <QCoreApplication>

#include <QEvent>
#include <QKeyEvent>
#include <QFocusEvent>

#include <iostream>


namespace Wdg
{


//
// Sliders_Pad
//

Sliders_Pad::Sliders_Pad (
	QWidget * parent_n ) :
QWidget ( parent_n ),
_proxies_groups ( 0 ),
_sp_layout ( _widgets_groups, _labels.vars(), this, &_labels ),
_labels ( _widgets_groups, _sp_style, this )
{
	setSizePolicy ( QSizePolicy::Expanding, QSizePolicy::Expanding );
	setContentsMargins ( 4, 4, 4, 7 );

	_sp_layout.widgets_groups_changed();
}


Sliders_Pad::~Sliders_Pad ( )
{
	set_proxies_groups ( 0 );
}


QSize
Sliders_Pad::minimumSizeHint ( ) const
{
	return _sp_layout.minimumSizeHint();
}


QSize
Sliders_Pad::sizeHint ( ) const
{
	return _sp_layout.sizeHint();
}


void
Sliders_Pad::set_pixmaps_buffers (
	const QSlider_Pixmaps_Buffers & sl_buff_n,
	const QCheck_Box_Pixmaps_Buffers & cb_buff_n )
{
	Sliders_Pad_Proxies_Group_List * lst ( proxies_groups() );
	set_proxies_groups ( 0 );

	_slider_buffers = sl_buff_n;
	_check_box_buffers = cb_buff_n;

	set_proxies_groups ( lst );
}


void
Sliders_Pad::clear_widgets_groups ( )
{
	if ( _widgets_groups.size() > 0 ) {
		for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {
			delete _widgets_groups[ii];
		}
		_widgets_groups.clear();
	}
}


void
Sliders_Pad::create_widgets_groups ( )
{
	if ( _proxies_groups == 0 ) {
		return;
	}

	if ( _proxies_groups->size() == 0 ) {
		return;
	}

	for ( int ii=0; ii < _proxies_groups->size(); ++ii ) {

		int style_id = -1;
		Sliders_Pad_Proxies_Group * sppg ( (*_proxies_groups)[ii] );
		Sliders_Pad_Widgets_Group * spwg ( new Sliders_Pad_Widgets_Group );

		spwg->set_proxies_group ( sppg );
		spwg->set_group_name ( sppg->group_name() );

		// Add widgets to the widgets groups
		for ( int jj=0; jj < sppg->proxies_sets().size(); ++jj ) {
			Sliders_Pad_Proxies_Set * spps ( sppg->proxies_set ( jj ) );
			Sliders_Pad_Widgets * spw ( new Sliders_Pad_Widgets );

			// Slider
			if ( spps->has_slider() ) {
				Sliders_Pad_Proxy_Slider * spp ( spps->slider_proxy() );

				Wdg::QSlider * qsl ( new Wdg::QSlider ( this, _slider_buffers ) );
				qsl->setRange ( spp->slider_min(), spp->slider_max() );
				qsl->setValue ( spp->slider_value() );
				qsl->setToolTip ( spp->tool_tip() );

				if ( spp->style_info() != 0 ) {
					Wdg::QSlider_Pixmaps_Config pix_cfg ( qsl->pixmaps_config() );
					pix_cfg.set_style_id ( spp->style_info()->style_id );
					style_id = spp->style_info()->style_id;

					if ( spp->style_info()->slider_has_minimum ) {
						pix_cfg.bg.bg_show_image = true;
						pix_cfg.bg.bg_tick_min_idx = spp->style_info()->slider_minimum_idx;
					}

					qsl->set_pixmaps_config ( pix_cfg );
				}

				connect ( qsl, SIGNAL ( valueChanged ( int) ),
					spp, SLOT ( set_slider_value ( int ) ) );

				connect ( spp, SIGNAL ( sig_slider_value_changed ( int) ),
					qsl, SLOT ( setValue ( int ) ) );

				qsl->installEventFilter ( this ); // Last filter to be passed
				qsl->installEventFilter ( spwg );
				qsl->installEventFilter ( sppg );
				qsl->installEventFilter ( spp ); // First filter to be passed

				spw->set_slider ( qsl );
			}

			// Button
			if ( spps->has_switch() ) {
				Sliders_Pad_Proxy_Switch * spp ( spps->switch_proxy() );

				Wdg::QCheck_Box * qcb (
					new Wdg::QCheck_Box ( this, _check_box_buffers ) );

				qcb->setChecked ( spp->switch_state() );
				qcb->setToolTip ( spp->tool_tip() );

				if ( spp->style_info() != 0 ) {
					Wdg::QCheck_Box_Pixmaps_Config pix_cfg ( qcb->pixmaps_config() );
					pix_cfg.set_style_id ( spp->style_info()->style_id );
					qcb->set_pixmaps_config ( pix_cfg );
				}

				connect ( qcb, SIGNAL ( toggled  ( bool ) ),
					spp, SLOT ( set_switch_state ( bool ) ) );

				connect ( spp, SIGNAL ( sig_switch_state_changed ( bool ) ),
					qcb, SLOT ( setChecked ( bool ) ) );

				qcb->installEventFilter ( this ); // Last filter to be passed
				qcb->installEventFilter ( spwg );
				qcb->installEventFilter ( sppg );
				qcb->installEventFilter ( spp ); // First filter to be passed

				spw->set_switch_wdg ( qcb );
			}

			if ( spw->has_any_widget() ) {
				spwg->append_widgets ( spw );
			} else {
				delete spw;
			}
		}

		if ( spwg->widgets().size() > 0 ) {

			// Adjust group style
			if ( ( _slider_buffers.painter() != 0 ) && ( style_id >= 0 ) ) {
				const QWidget_Style * style (
					_slider_buffers.painter()->style ( style_id ) );
				if ( style != 0 ) {
					spwg->pen_stem_normal_ref().setColor ( style->border_col_normal );
					spwg->pen_stem_focus_ref().setColor ( style->border_col_focus );
					spwg->col_text_ref() = style->col_fg;
				}
			}

			spwg->set_all_visible ( true );
			_widgets_groups.append ( spwg );
		} else {
			delete spwg;
		}

	}

}


void
Sliders_Pad::set_proxies_groups (
	Sliders_Pad_Proxies_Group_List * list_n )
{
	if ( list_n != _proxies_groups ) {
		//std::cout << "Sliders_Pad::set_proxies_groups " << list_n << "\n";

		clear_widgets_groups();

		_proxies_groups = list_n;

		create_widgets_groups();

		_sp_layout.widgets_groups_changed();
		updateGeometry();
		adjustSize();

		{
			QEvent ev ( QEvent ( QEvent::LayoutRequest ) );
			QCoreApplication::sendEvent ( this, &ev );
		}
	}
}


void
Sliders_Pad::update_widgets_graphics ( )
{
	const QRect con_rect ( contentsRect() );
	const unsigned int vspace ( _sp_layout.spacing_vertical() );

	for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {

		Sliders_Pad_Widgets_Group * spwg ( _widgets_groups[ii] );
		QPainterPath & ppath ( spwg->ppath_bottom() );
		ppath = QPainterPath();

		// No lines required
		if ( ( spwg->num_sliders() <= 1 ) &&
			( spwg->num_switches() == 0 ) )
		{
			continue;
		}


		// Declaration of variables that may be needed later
		bool draw_sl_bridge ( false );
		bool draw_sw_bridge ( false );

		const unsigned int sl_max_idx ( spwg->num_sliders() - 1 );
		const unsigned int sw_max_idx ( spwg->num_switches() - 1 );
		qreal c_off ( _sp_style.stem_corner_indent ); // Corner offset
		qreal x_mid;
		qreal y_mid;

		if ( ( spwg->group_width() % 2 ) != 0 ) {
			x_mid = spwg->group_width();
		} else {
			x_mid = spwg->group_width() - 1;
		}
		if ( ( vspace % 2 ) != 0 ) {
			y_mid = vspace;
		} else {
			y_mid = vspace - 1;
		}
		x_mid = x_mid / qreal ( 2.0 );
		y_mid = y_mid / qreal ( 2.0 );

		// Slider Bridge lines required
		if ( spwg->num_sliders() > 1 ) {
			draw_sl_bridge = true;
		}

		// Vertical lines from the sliders downwards to the switches required
		if ( spwg->num_sliders() == spwg->num_switches() ) {
			for ( unsigned int jj=0; jj < spwg->num_sliders(); ++jj ) {
				const Sliders_Pad_Widgets * spw ( spwg->widgets ( jj ) );

				qreal cc ( spw->pos() - spwg->group_pos() );
				cc += _sp_style.calc_center (
					spw->width(), jj, spwg->num_sliders() );

				ppath.moveTo ( cc, 0 );
				ppath.lineTo ( cc, vspace );
			}
		}

		// Single switch multiple sliders
		if ( ( spwg->num_sliders() > 1 ) &&
			( spwg->num_switches() == 1 ) )
		{
			// Draw downward from the bridge line to the single switch
			ppath.moveTo ( x_mid, y_mid );
			ppath.lineTo ( x_mid, vspace );
		}

		// Single slider multiple switches
		if ( ( spwg->num_sliders() == 1 ) &&
			( spwg->num_switches() > 1 ) )
		{
			// Draw downward from the single slider to the bridge line
			ppath.moveTo ( x_mid, 0 );
			ppath.lineTo ( x_mid, y_mid );

			draw_sw_bridge = true;
		}


		// Draw a slider connecting bridge line
		if ( draw_sl_bridge ) {

			// Draw bridge line from the leftmost slider to the rightmost
			{
				const Sliders_Pad_Widgets * spw_b ( spwg->widgets ( 0 ) );
				const Sliders_Pad_Widgets * spw_e ( spwg->widgets ( sl_max_idx ) );

				qreal cc_min ( spw_b->pos() - spwg->group_pos() );
				qreal cc_max ( spw_e->pos() - spwg->group_pos() );

				cc_min += _sp_style.calc_center (
					spw_b->width(), 0, spwg->num_sliders() );

				cc_max += _sp_style.calc_center (
					spw_b->width(), sl_max_idx, spwg->num_sliders() );

				ppath.moveTo ( cc_min, 0 );
				ppath.lineTo ( cc_min, y_mid - c_off );
				ppath.lineTo ( cc_min + c_off, y_mid );
				ppath.lineTo ( cc_max - c_off, y_mid );
				ppath.lineTo ( cc_max, y_mid - c_off );
				ppath.lineTo ( cc_max, 0 );
			}

			// Draw vertical lines from the inner sliders down to the bridge line
			for ( unsigned int jj=1; jj < sl_max_idx; ++jj ) {
				const Sliders_Pad_Widgets * spw ( spwg->widgets ( jj ) );
				qreal cc ( spw->pos() - spwg->group_pos() );
				cc += _sp_style.calc_center (
					spw->width(), jj, spwg->num_sliders() );

				ppath.moveTo ( cc, 0 );
				ppath.lineTo ( cc, y_mid );
			}

		}


		// Draw a switches connecting bridge line
		if ( draw_sw_bridge ) {

			// Draw bridge line from the leftmost switch to the rightmost
			{
				const Sliders_Pad_Widgets * spw_b ( spwg->widgets ( 0 ) );
				const Sliders_Pad_Widgets * spw_e ( spwg->widgets ( sw_max_idx ) );

				qreal cc_min ( spw_b->pos() - spwg->group_pos() );
				qreal cc_max ( spw_e->pos() - spwg->group_pos() );

				cc_min += _sp_style.calc_center (
					spw_b->width(), 0, spwg->num_switches() );

				cc_max += _sp_style.calc_center (
					spw_b->width(), sw_max_idx, spwg->num_switches() );

				ppath.moveTo ( cc_min, vspace );
				ppath.lineTo ( cc_min, y_mid + c_off );
				ppath.lineTo ( cc_min + c_off, y_mid );
				ppath.lineTo ( cc_min - c_off, y_mid );
				ppath.lineTo ( cc_max, y_mid + c_off );
				ppath.lineTo ( cc_max, vspace );
			}


			// Draw vertical lines down from the bridge line to the inner switches
			for ( unsigned int jj=1; jj < sw_max_idx; ++jj ) {
				const Sliders_Pad_Widgets * spw ( spwg->widgets ( jj ) );

				qreal cc ( spw->pos() - spwg->group_pos() );
				cc += _sp_style.calc_center (
					spw->width(), jj, spwg->num_switches() );

				ppath.moveTo ( cc, y_mid );
				ppath.lineTo ( cc, vspace );
			}
		}

	}

	_labels.update_widgets_graphics();
}


void
Sliders_Pad::update_focus_info ( )
{
	_focus_info.has_focus = false;
	for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {
		Sliders_Pad_Widgets_Group * spwg ( _widgets_groups[ii] );
		Sliders_Pad_Widgets * spw ( spwg->focus_widgets() );

		if ( spw != 0 ) {
			_focus_info.has_focus = true;
			_focus_info.group_idx   = ii;
			_focus_info.widgets_idx = spwg->focus_widgets_idx();
			break;
		}
	}
}


void
Sliders_Pad::set_focus_proxy (
	unsigned int proxy_group_idx_n,
	unsigned char proxy_idx_n )
{
	const unsigned int num_groups ( _widgets_groups.size() );
	if ( proxy_group_idx_n < num_groups ) {
		Sliders_Pad_Widgets_Group * spwg (
			_widgets_groups[ proxy_group_idx_n ] );

		QWidget * wdg ( 0 );

		// Select the slider of given widget set
		if ( int ( proxy_idx_n ) < spwg->widgets().size() ) {
			wdg = spwg->widgets ( proxy_idx_n )->slider();
		}

		// No widget found so far - Take first slider then
		if ( wdg == 0 ) {
			for ( int ii=0; ii < spwg->widgets().size(); ++ii ) {
				Sliders_Pad_Widgets * spw ( spwg->widgets ( ii ) );
				if ( spw->slider() != 0 ) {
					wdg = spw->slider();
					break;
				}
			}
		}

		if ( wdg != 0 ) {
			wdg->setFocus();
		}
	}
}


bool
Sliders_Pad::eventFilter (
	QObject * watched,
	QEvent * event )
{
	QWidget * wdg ( dynamic_cast < QWidget * > ( watched ) );
	if ( wdg != 0 ) {
		QFocusEvent * fev ( dynamic_cast < QFocusEvent * > ( event ) );
		if ( ( fev != 0 ) ) {
			//std::cout << "Sliders_Pad::eventFilter: QFocusEvent" << "\n";
			if ( fev->lostFocus() ) {
				_focus_info.clear();
			} else if ( fev->gotFocus() ) {
				update_focus_info();
			}
			_labels.update();
			emit sig_focus_changed();
			return false;
		}
	}

	return false;
}


bool
Sliders_Pad::event (
	QEvent * event )
{
	bool res ( QWidget::event ( event ) );
	bool do_layout ( false );
	switch ( event->type() ) {

		case QEvent::LayoutRequest:
			//std::cout << "Sliders_Pad::event QEvent::LayoutRequest" << "\n";
			do_layout = true;
			break;

		case QEvent::Resize:
			//std::cout << "Sliders_Pad::event QEvent::Resize" << "\n";
			do_layout = true;
			break;

		default:
			break;
	}

	if ( do_layout ) {
		_sp_layout.update_widgets_geometry();
		update_widgets_graphics();
	}

	return res;
}


void
Sliders_Pad::paintEvent (
	QPaintEvent * event )
{
	QWidget::paintEvent ( event );
	paint_decoration ( event );
}


void
Sliders_Pad::paint_decoration (
	QPaintEvent * )
{
	qreal pa_left;
	qreal pa_top;

	{
		const QRect con_rect ( contentsRect() );
		pa_left = con_rect.left();
		pa_top = ( con_rect.top() + con_rect.height()
			- _sp_layout.switch_height() - _sp_layout.spacing_vertical() );
	}

	QPainter painter ( this );
	painter.setRenderHints (
		QPainter::Antialiasing |
		QPainter::TextAntialiasing |
		QPainter::SmoothPixmapTransform );

	painter.setBrush ( Qt::NoBrush );

	for ( int ii=0; ii < _widgets_groups.size(); ++ii ) {
		const Sliders_Pad_Widgets_Group * spwg ( _widgets_groups[ii] );
		{
			QTransform trans (
				1, 0, 0,
				0, 1, 0,
				pa_left + spwg->group_pos(), pa_top, 1 );
			painter.setTransform ( trans );
		}
		painter.setPen ( spwg->pen_stem_normal() );
		painter.drawPath ( spwg->ppath_bottom() );
	}
}


} // End of namespace

