//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_sliders_pad_hpp__
#define __INC_sliders_pad_hpp__

#include "pad_focus_info.hpp"
#include "sliders_pad_layout.hpp"
#include "sliders_pad_style.hpp"
#include "sliders_pad_labels.hpp"
#include "sliders_pad_widgets_group.hpp"

#include <QWidget>
#include <QFrame>
#include <QScrollArea>
#include <QScrollBar>


namespace Wdg
{


///
/// @brief Sliders_Pad
///
class Sliders_Pad :
	public QWidget
{
	Q_OBJECT

	// Public methods
	public:

	Sliders_Pad (
		QWidget * parent_n = 0 );

	~Sliders_Pad ( );


	QSize
	minimumSizeHint ( ) const;

	QSize
	sizeHint ( ) const;


	// Proxies groups

	void
	set_proxies_groups (
		Sliders_Pad_Proxies_Group_List * list_n );

	Sliders_Pad_Proxies_Group_List *
	proxies_groups ( ) const;


	// Widgets groups

	const Sliders_Pad_Widgets_Group_List &
	widgets_groups ( ) const;


	// Pixmaps buffers

	void
	set_pixmaps_buffers (
		const QSlider_Pixmaps_Buffers & sl_buff_n,
		const QCheck_Box_Pixmaps_Buffers & cb_buff_n );


	// Focus info

	const Pad_Focus_Info &
	focus_info ( ) const;

	void
	set_focus_proxy (
		unsigned int proxy_group_idx_n,
		unsigned char proxy_idx_n = 0 );


	// Labels widget

	const Sliders_Pad_Labels &
	labels ( ) const;

	Sliders_Pad_Labels &
	labels ( );


	// Public signals
	signals:

	void
	sig_focus_changed ( );


	// Protected methods
	protected:

	void
	clear_widgets_groups ( );

	void
	create_widgets_groups ( );

	void
	update_focus_info ( );

	void
	update_widgets_graphics ( );


	// Event methods

	bool
	eventFilter (
		QObject * watched,
		QEvent * event );

	bool
	event (
		QEvent * event );

	void
	paintEvent (
		QPaintEvent * event );

	void
	paint_decoration (
		QPaintEvent * event );


	// Private attributes
	private:

	Sliders_Pad_Proxies_Group_List * _proxies_groups;

	Sliders_Pad_Widgets_Group_List _widgets_groups;


	QSlider_Pixmaps_Buffers _slider_buffers;

	QCheck_Box_Pixmaps_Buffers _check_box_buffers;


	Sliders_Pad_Style _sp_style;

	Sliders_Pad_Layout _sp_layout;

	Sliders_Pad_Labels _labels;


	Pad_Focus_Info _focus_info;
};


inline
const Sliders_Pad_Widgets_Group_List &
Sliders_Pad::widgets_groups ( ) const
{
	return _widgets_groups;
}


inline
Sliders_Pad_Proxies_Group_List *
Sliders_Pad::proxies_groups ( ) const
{
	return _proxies_groups;
}


inline
const Pad_Focus_Info &
Sliders_Pad::focus_info ( ) const
{
	return _focus_info;
}


inline
const Sliders_Pad_Labels &
Sliders_Pad::labels ( ) const
{
	return _labels;
}


inline
Sliders_Pad_Labels &
Sliders_Pad::labels ( )
{
	return _labels;
}


} // End of namespace


#endif


