//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_sliders_pad_layout_hpp__
#define __INC_sliders_pad_layout_hpp__

#include <QPen>
#include <QObject>
#include <QWidget>
#include <QLayout>

#include "pixmaps_hub.hpp"
#include "sliders_pad_widgets_group.hpp"
#include "sliders_pad_labels_vars.hpp"


namespace Wdg
{


///
/// @brief Sliders_Pad_Layout
///
class Sliders_Pad_Layout
{
	// Public methods
	public:

	Sliders_Pad_Layout (
		Sliders_Pad_Widgets_Group_List & groups_n,
		Sliders_Pad_Labels_Vars & labels_vars_n,
		QWidget * parent_n,
		QWidget * labels_n );


	//
	// Weight methods
	//

	void
	normalize_weights (
		unsigned long * weights_n );

	void
	set_weights (
		unsigned long * weights_n,
		unsigned long value_n ) const;

	///
	/// @brief Calculates the normalized difference between requested weights and current weights
	///
	void
	calc_weights_delta (
		long * weights_delta_n,
		const unsigned long * weights_requested_n,
		const unsigned long * weights_currently_n );

	void
	calc_weights_delta_sorted (
		unsigned int * sorting_n,
		const long * weights_deltas_n );


	unsigned long
	sum_up (
		const unsigned long * weights_n ) const;


	//
	// Variable reader
	//

	int
	spacing_vertical ( ) const;

	int
	switch_height ( ) const;


	//
	// Size hints
	//

	QSize
	minimumSizeHint ( ) const;

	QSize
	sizeHint ( ) const;


	//
	// Pad interface
	//

	void
	widgets_groups_changed ( );

	void
	update_widgets_geometry ( );



	// Protected methods
	protected:

	void
	update_basic_sizes ( );

	void
	update_type_count ( );

	void
	update_size_hints ( );

	unsigned int
	label_str_length_px_max (
		const QString & str_n ) const;

	void
	calc_label_angle (
		int full_width_n );

	int
	calc_label_max_x (
		int idx_n );

	int
	calc_labels_max_x ( );

	int
	labels_height_hint ( ) const;

	void
	calc_column_widths (
		const int width_n,
		const int height_n );

	void
	calc_column_sizes (
		const QRect & rect_n );



	// Private attributes;
	private:

	Sliders_Pad_Widgets_Group_List & _widgets_groups;

	QWidget * _parent;

	QWidget * _labels;

	Sliders_Pad_Labels_Vars & _labels_vars;

	unsigned int _type_count[4];

	unsigned int _group_min_width;


	int _min_spacing_horizontal_major;

	int _min_spacing_horizontal_minor;

	int _spacing_vertical;


	int _switch_height_min;

	int _slider_width_min;


	int _input_area_min_width;

	int _input_area_min_height;


	const unsigned long _weight_normal;
};


inline
void
Sliders_Pad_Layout::set_weights (
	unsigned long * weights_n,
	unsigned long value_n ) const
{
	for ( int ii=0; ii < 4; ++ii ) {
		weights_n[ii] = value_n;
	}
}


inline
unsigned long
Sliders_Pad_Layout::sum_up (
	const unsigned long * weights_n ) const
{
	unsigned long res ( 0 );
	for ( int ii=0; ii < 4; ++ii ) {
		res += weights_n[ii];
	}
	return res;
}


inline
int
Sliders_Pad_Layout::spacing_vertical ( ) const
{
	return _spacing_vertical;
}


inline
int
Sliders_Pad_Layout::switch_height ( ) const
{
	return _switch_height_min;
}


} // End of namespace


#endif


