//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_sliders_pad_proxies_group_hpp__
#define __INC_sliders_pad_proxies_group_hpp__

#include <QObject>
#include <QList>

#include "sliders_pad_proxies_set.hpp"


namespace Wdg
{


class Sliders_Pad_Proxies_Group :
	public QObject
{
	Q_OBJECT


	// Public methods
	public:

	Sliders_Pad_Proxies_Group (
		QObject * parent_n = 0 );

	virtual
	~Sliders_Pad_Proxies_Group ( );


	const Sliders_Pad_Proxies_Set_List &
	proxies_sets ( ) const;

	void
	append_proxies_set (
		Sliders_Pad_Proxies_Set * proxies_set_n );

	void
	clear_proxies_sets ( );

	Sliders_Pad_Proxies_Set *
	proxies_set (
		int idx_n );


	// Group name

	const QString &
	group_name ( ) const;

	void
	set_group_name (
		const QString & name_n );


	// Tool tip

	const QString &
	tool_tip ( ) const;

	void
	set_tool_tip (
		const QString & tip_n );


	unsigned int
	num_sliders ( ) const;

	unsigned int
	num_switches ( ) const;


	// Private attributes
	private:

	Sliders_Pad_Proxies_Set_List _proxies_sets;

	QString _group_name;

	QString _tool_tip;

	unsigned int _num_sliders;

	unsigned int _num_switches;
};


inline
const Sliders_Pad_Proxies_Set_List &
Sliders_Pad_Proxies_Group::proxies_sets ( ) const
{
	return _proxies_sets;
}


inline
void
Sliders_Pad_Proxies_Group::append_proxies_set (
	Sliders_Pad_Proxies_Set * proxies_set_n )
{
	_proxies_sets.append ( proxies_set_n );
	if ( proxies_set_n->has_slider() != 0 ) {
		++_num_sliders;
	}
	if ( proxies_set_n->has_switch() != 0 ) {
		++_num_switches;
	}
}


inline
Sliders_Pad_Proxies_Set *
Sliders_Pad_Proxies_Group::proxies_set (
	int idx_n )
{
	return _proxies_sets[idx_n];
}


inline
const QString &
Sliders_Pad_Proxies_Group::group_name ( ) const
{
	return _group_name;
}


inline
const QString &
Sliders_Pad_Proxies_Group::tool_tip ( ) const
{
	return _tool_tip;
}


inline
unsigned int
Sliders_Pad_Proxies_Group::num_sliders ( ) const
{
	return _num_sliders;
}


inline
unsigned int
Sliders_Pad_Proxies_Group::num_switches ( ) const
{
	return _num_switches;
}


typedef QList < Sliders_Pad_Proxies_Group * > Sliders_Pad_Proxies_Group_List;


} // End of namespace


#endif


