//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "sliders_pad_widgets_group.hpp"

#include <QKeyEvent>
#include <QFocusEvent>

#include <QCoreApplication>

#include <iostream>


namespace Wdg
{


//
// Sliders_Pad_Widgets_Group
//

Sliders_Pad_Widgets_Group::Sliders_Pad_Widgets_Group ( ) :
_group_pos ( 0 ),
_group_width ( 0 ),
_focus_widgets ( 0 ),
_focus_widgets_idx ( 0 )
{
	_pen_stem_normal.setWidthF ( 1 );
	_pen_stem_normal.setCapStyle ( Qt::FlatCap );
	_pen_stem_normal.setJoinStyle ( Qt::BevelJoin );

	_pen_stem_focus.setWidthF ( 1 );
	_pen_stem_focus.setCapStyle ( Qt::FlatCap );
	_pen_stem_focus.setJoinStyle ( Qt::BevelJoin );
}


Sliders_Pad_Widgets_Group::~Sliders_Pad_Widgets_Group ( )
{
	clear_widgets();
}


void
Sliders_Pad_Widgets_Group::clear_widgets ( )
{
	for ( int ii=0; ii < _widgets.size(); ++ ii ) {
		delete _widgets[ii];
	}
	_widgets.clear();
}


void
Sliders_Pad_Widgets_Group::append_widgets (
	Sliders_Pad_Widgets * wdgs_n )
{
	_widgets.append ( wdgs_n );
}


void
Sliders_Pad_Widgets_Group::set_proxies_group (
	Sliders_Pad_Proxies_Group * sppg_n )
{
	_proxies_group = sppg_n;
}


void
Sliders_Pad_Widgets_Group::set_group_name (
	const QString & name_n )
{
	_group_name = name_n;
}


void
Sliders_Pad_Widgets_Group::set_all_visible (
	bool flag_n )
{
	for ( int ii=0; ii < _widgets.size(); ++ii ) {
		_widgets[ii]->set_all_visible ( flag_n );
	}
}


Sliders_Pad_Widgets::Slider_Type *
Sliders_Pad_Widgets_Group::focus_slider ( ) const
{
	Sliders_Pad_Widgets::Slider_Type * res ( 0 );
	if ( _focus_widgets != 0 ) {
		if ( _focus_widgets->slider() != 0 ) {
			if (  _focus_widgets->slider()->hasFocus() ) {
				res = _focus_widgets->slider();
			}
		}
	}

	return res;
}


Sliders_Pad_Widgets::Switch_Type *
Sliders_Pad_Widgets_Group::focus_switch ( ) const
{
	Sliders_Pad_Widgets::Switch_Type * res ( 0 );
	if ( _focus_widgets != 0 ) {
		if ( _focus_widgets->switch_wdg() != 0 ) {
			if (  _focus_widgets->switch_wdg()->hasFocus() ) {
				res = _focus_widgets->switch_wdg();
			}
		}
	}

	return res;
}


bool
Sliders_Pad_Widgets_Group::eventFilter (
	QObject * watched,
	QEvent * event )
{
	Sliders_Pad_Widgets::Slider_Type * sl (
		dynamic_cast < Sliders_Pad_Widgets::Slider_Type * > ( watched ) );
	//Sliders_Pad_Widgets::Switch_Type * sw (
	//	dynamic_cast < Sliders_Pad_Widgets::Switch_Type * > ( watched ) );

	QKeyEvent * kev ( dynamic_cast < QKeyEvent * > ( event ) );

	if ( ( kev != 0 ) && ( sl != 0 ) ) {

		//std::cout << "Sliders_Pad_Widgets_Group::eventFilter: QKeyEvent" << "\n";

		if ( ( num_switches() == 0 ) ||
			( kev->key() != Qt::Key_Space ) )
		{
			return false;
		}

		if ( num_sliders() == 1 )
		{
			// Pass event to all switches
			for ( unsigned int ii=0; ii < num_switches(); ++ii ) {
				QKeyEvent ev_pass ( *kev );
				QCoreApplication::sendEvent ( switch_wdg ( ii ), &ev_pass );
			}

		} else {

			QWidget * wdg ( 0 );
			if ( num_switches() == 1 ) {
				// Pass event to the only switch
				wdg = switch_wdg ( 0 );
			} else {
				// Pass event to the matching switch
				for ( int ii=0; ii < widgets().size(); ++ii ) {
					Sliders_Pad_Widgets * spw ( widgets ( ii ) );
					if ( spw->slider() == watched ) {
						wdg = spw->switch_wdg();
						break;
					}
				}
			}

			if ( wdg != 0 ) {
				QKeyEvent ev_pass ( *kev );
				QCoreApplication::sendEvent ( wdg, &ev_pass );
			}
		}

		return false;
	}


	QFocusEvent * fev ( dynamic_cast < QFocusEvent * > ( event ) );
	if ( fev != 0 ) {

		_focus_widgets = 0;
		_focus_widgets_idx = 0;

		if ( fev->gotFocus () ) {

			// Find focus widgets
			int ii ( 0 );
			for ( ; ii < _widgets.size(); ++ii ) {
				Sliders_Pad_Widgets * spw ( _widgets[ii] );
				if ( spw->slider() != 0 ) {
					if ( spw->slider()->hasFocus() ) {
						_focus_widgets = spw;
						break;
					}
				}
				if ( spw->switch_wdg() != 0 ) {
					if ( spw->switch_wdg()->hasFocus() ) {
						_focus_widgets = spw;
						break;
					}
				}
			}

			if ( _focus_widgets != 0 ) {
				_focus_widgets_idx = ii;
			}

		}
		return false;
	}

	return false;
}


} // End of namespace

