//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_sliders_pad_widgets_group_hpp__
#define __INC_sliders_pad_widgets_group_hpp__

#include <QObject>
#include <QList>

#include <QRectF>
#include <QTransform>
#include <QPainterPath>


#include "sliders_pad_widgets.hpp"
#include "sliders_pad_proxies_group.hpp"


namespace Wdg
{


///
/// @brief Sliders_Pad_Widgets_Group
///
class Sliders_Pad_Widgets_Group :
	public QObject
{
	Q_OBJECT

	// Public typedefs
	public:

	typedef QList < Sliders_Pad_Widgets * > Widgets_List;


	// Public methods
	public:

	Sliders_Pad_Widgets_Group ( );

	~Sliders_Pad_Widgets_Group ( );


	// Proxy group

	Sliders_Pad_Proxies_Group *
	proxies_group ( ) const;

	void
	set_proxies_group (
		Sliders_Pad_Proxies_Group * sppg_n );


	// Widgets list

	void
	clear_widgets ( );

	const Widgets_List &
	widgets ( ) const;

	void
	append_widgets (
		Sliders_Pad_Widgets * wdgs_n );


	const Sliders_Pad_Widgets *
	widgets (
		int idx_n ) const;

	Sliders_Pad_Widgets *
	widgets (
		int idx_n );



	Sliders_Pad_Widgets::Slider_Type *
	slider (
		int idx_n ) const;

	Sliders_Pad_Widgets::Switch_Type *
	switch_wdg (
		int idx_n ) const;


	// Group name

	const QString &
	group_name ( ) const;

	void
	set_group_name (
		const QString & name_n );


	// Widget count

	unsigned int
	num_sliders ( ) const;

	unsigned int
	num_switches ( ) const;


	// Group geometry

	int
	group_pos ( ) const;

	void
	set_group_pos (
		int value_n );


	int
	group_width ( ) const;

	void
	set_group_width (
		int value_n );


	Sliders_Pad_Widgets *
	focus_widgets ( ) const;

	unsigned int
	focus_widgets_idx ( ) const;


	// Mass adjusting

	void
	set_all_visible (
		bool flag_n );

	Sliders_Pad_Widgets::Slider_Type *
	focus_slider ( ) const;

	Sliders_Pad_Widgets::Switch_Type *
	focus_switch ( ) const;


	// Graphics stuff

	const QPen &
	pen_stem_normal ( ) const;

	QPen &
	pen_stem_normal_ref ( );


	const QPen &
	pen_stem_focus ( ) const;

	QPen &
	pen_stem_focus_ref ( );


	const QColor &
	col_text ( ) const;

	QColor &
	col_text_ref ( );


	const QTransform &
	label_transform ( ) const;

	const QTransform &
	label_transform_inv ( ) const;

	void
	set_label_transform (
		const QTransform & trans_n );

	const QRectF &
	label_rect ( ) const;

	void
	set_label_rect (
		const QRectF & rect_n );

	const QPainterPath &
	ppath_top ( ) const;

	QPainterPath &
	ppath_top ( );


	const QPainterPath &
	ppath_bottom ( ) const;

	QPainterPath &
	ppath_bottom ( );


	// Ṕrotected methods
	protected:

	bool
	eventFilter (
		QObject * watched,
		QEvent * event );


	// Private attributes
	private:

	Sliders_Pad_Proxies_Group * _proxies_group;

	Widgets_List _widgets;

	QString _group_name;

	QString _tool_tip;


	// Layout helpers

	int _group_pos;

	int _group_width;


	// Graphics

	QPen _pen_stem_normal;

	QPen _pen_stem_focus;

	QColor _col_text;


	QTransform _label_transform;

	QTransform _label_transform_inv;

	QRectF _label_rect;

	QPainterPath _ppath_top;

	QPainterPath _ppath_bottom;


	// Focus widgets

	Sliders_Pad_Widgets * _focus_widgets;

	unsigned int _focus_widgets_idx;
};


inline
Sliders_Pad_Proxies_Group *
Sliders_Pad_Widgets_Group::proxies_group ( ) const
{
	return _proxies_group;
}


inline
const Sliders_Pad_Widgets_Group::Widgets_List &
Sliders_Pad_Widgets_Group::widgets ( ) const
{
	return _widgets;
}


inline
const Sliders_Pad_Widgets *
Sliders_Pad_Widgets_Group::widgets (
	int idx_n ) const
{
	return _widgets[idx_n];
}


inline
Sliders_Pad_Widgets *
Sliders_Pad_Widgets_Group::widgets (
	int idx_n )
{
	return _widgets[idx_n];
}


inline
Sliders_Pad_Widgets::Slider_Type *
Sliders_Pad_Widgets_Group::slider (
	int idx_n ) const
{
	return _widgets[idx_n]->slider();
}


inline
Sliders_Pad_Widgets::Switch_Type *
Sliders_Pad_Widgets_Group::switch_wdg (
	int idx_n ) const
{
	return _widgets[idx_n]->switch_wdg();
}


inline
const QString &
Sliders_Pad_Widgets_Group::group_name ( ) const
{
	return _group_name;
}


inline
unsigned int
Sliders_Pad_Widgets_Group::num_sliders ( ) const
{
	return proxies_group()->num_sliders();
}


inline
unsigned int
Sliders_Pad_Widgets_Group::num_switches ( ) const
{
	return proxies_group()->num_switches();
}


inline
int
Sliders_Pad_Widgets_Group::group_pos ( ) const
{
	return _group_pos;
}


inline
void
Sliders_Pad_Widgets_Group::set_group_pos (
	int value_n )
{
	_group_pos = value_n;
}


inline
int
Sliders_Pad_Widgets_Group::group_width ( ) const
{
	return _group_width;
}


inline
void
Sliders_Pad_Widgets_Group::set_group_width (
	int value_n )
{
	_group_width = value_n;
}


inline
Sliders_Pad_Widgets *
Sliders_Pad_Widgets_Group::focus_widgets ( ) const
{
	return _focus_widgets;
}


inline
unsigned int
Sliders_Pad_Widgets_Group::focus_widgets_idx ( ) const
{
	return _focus_widgets_idx;
}


// Graphics stuff

inline
const QPen &
Sliders_Pad_Widgets_Group::pen_stem_normal ( ) const
{
	return _pen_stem_normal;
}


inline
QPen &
Sliders_Pad_Widgets_Group::pen_stem_normal_ref ( )
{
	return _pen_stem_normal;
}


inline
const QPen &
Sliders_Pad_Widgets_Group::pen_stem_focus ( ) const
{
	return _pen_stem_focus;
}


inline
QPen &
Sliders_Pad_Widgets_Group::pen_stem_focus_ref ( )
{
	return _pen_stem_focus;
}


inline
const QColor &
Sliders_Pad_Widgets_Group::col_text ( ) const
{
	return _col_text;
}


inline
QColor &
Sliders_Pad_Widgets_Group::col_text_ref ( )
{
	return _col_text;
}



inline
const QTransform &
Sliders_Pad_Widgets_Group::label_transform ( ) const
{
	return _label_transform;
}


inline
const QTransform &
Sliders_Pad_Widgets_Group::label_transform_inv ( ) const
{
	return _label_transform_inv;
}



inline
void
Sliders_Pad_Widgets_Group::set_label_transform (
	const QTransform & trans_n )
{
	_label_transform = trans_n;
	_label_transform_inv = _label_transform.inverted();
}


inline
const QRectF &
Sliders_Pad_Widgets_Group::label_rect ( ) const
{
	return _label_rect;
}


inline
void
Sliders_Pad_Widgets_Group::set_label_rect (
	const QRectF & rect_n )
{
	_label_rect = rect_n;
}


inline
const QPainterPath &
Sliders_Pad_Widgets_Group::ppath_top ( ) const
{
	return _ppath_top;
}


inline
QPainterPath &
Sliders_Pad_Widgets_Group::ppath_top ( )
{
	return _ppath_top;
}


inline
const QPainterPath &
Sliders_Pad_Widgets_Group::ppath_bottom ( ) const
{
	return _ppath_bottom;
}


inline
QPainterPath &
Sliders_Pad_Widgets_Group::ppath_bottom ( )
{
	return _ppath_bottom;
}



typedef QList < Sliders_Pad_Widgets_Group * > Sliders_Pad_Widgets_Group_List;


} // End of namespace


#endif

