//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_qsnd_control_info_hpp__
#define __INC_qsnd_control_info_hpp__


#include <QString>
#include <QVector>
#include "alsa.hpp"


namespace QSnd
{


///
/// Snd_Control_Argument
///
class Snd_Control_Argument
{
	// Public methods
	public:

	Snd_Control_Argument (
		const QString & name_n = QString(),
		const QString & value_n = QString(),
		const QString & type_n = QString ( "string" ) );

	void
	clear ( );


	// Public attributes

	QString arg_name;
	QString arg_value;

	QString arg_type;

	Snd_Control_Argument &
	operator = (
		const Snd_Control_Argument & cinfo_n );
};


///
/// Snd_Control_Info
///
class Snd_Control_Info
{
	// Public methods
	public:

	Snd_Control_Info (
		const QString & ctl_id_n = QString() );

	void
	clear ( );

	bool
	is_clear ( ) const;


	// Control name

	const QString &
	ctl_name ( ) const;

	void
	set_ctl_name (
		const QString & name_n );


	void
	set_ctl_id (
		const QString & id_n );


	// Control arguments

	int
	num_args ( ) const;

	const Snd_Control_Argument &
	arg (
		int idx_n ) const;

	void
	append_arg (
		const Snd_Control_Argument & arg_n );

	void
	set_arg_value (
		int idx_n,
		const QString & value_n );

	void
	set_arg_value (
		const QString & arg_name_n,
		const QString & value_n );



	// Control id string

	const QString &
	ctl_id ( ) const;


	Snd_Control_Info &
	operator = (
		const Snd_Control_Info & cinfo_n );

	bool
	operator == (
		const Snd_Control_Info & cinfo_n ) const;

	bool
	operator != (
		const Snd_Control_Info & cinfo_n ) const;


	// Protected methods
	protected:

	void
	update_ctl_id ( );


	// Private attributes
	private:

	QString _ctl_name;

	QVector < Snd_Control_Argument > _arguments;

	QString _ctl_id;
};


inline
bool
Snd_Control_Info::is_clear ( ) const
{
	return _ctl_name.isEmpty();
}


inline
const QString &
Snd_Control_Info::ctl_name ( ) const
{
	return _ctl_name;
}


inline
int
Snd_Control_Info::num_args ( ) const
{
	return _arguments.size();
}


inline
const Snd_Control_Argument &
Snd_Control_Info::arg (
	int idx_n ) const
{
	return _arguments[idx_n];
}


inline
const QString &
Snd_Control_Info::ctl_id ( ) const
{
	return _ctl_id;
}


} // End of namespace


#endif


