//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_snd_controls_model_hpp__
#define __INC_snd_controls_model_hpp__

#include <QStandardItemModel>
#include <QVector>


#include "snd_card_info.hpp"
#include "snd_control_info.hpp"


namespace QSnd
{


///
/// Snd_Controls_Model
///
class Snd_Controls_Model :
	public QStandardItemModel
{
	Q_OBJECT;

	// Public methods
	public:

	Snd_Controls_Model ( );

	~Snd_Controls_Model ( );


	// Base items access

	const QStandardItem *
	item_cards ( ) const;

	const QStandardItem *
	item_plugs ( ) const;


	// Init / close

	void
	update_data ( );


	// Cards access

	int
	num_cards ( ) const;

	const QSnd::Snd_Card_Info *
	card_info (
		int index_n ) const;


	// Controls access

	int
	num_controls ( ) const;

	const QSnd::Snd_Control_Info *
	ctl_info (
		const QModelIndex & idx_n ) const;

	QModelIndex
	ctl_info_index (
		const QSnd::Snd_Control_Info & ctl_info_n ) const;


	// Default values

	const QSnd::Snd_Control_Info &
	default_ctl ( ) const;

	QModelIndex
	default_index ( ) const;


	// Protected methods
	protected:

	void
	load_cards ( );

	void
	load_plugins ( );

	void
	clear_cards_and_plugins ( );


	// Private attributes
	private:

	QStandardItem * _item_cards;

	QStandardItem * _item_plugs;

	QVector < QSnd::Snd_Card_Info * > _card_infos;

	QVector < QSnd::Snd_Control_Info > _controls_cards;

	QVector < QSnd::Snd_Control_Info > _controls_plugs;


	QModelIndex _default_index;

	QSnd::Snd_Control_Info _default_ctl;

	int _default_card_idx;
};


inline
const QStandardItem *
Snd_Controls_Model::item_cards ( ) const
{
	return _item_cards;
}


inline
const QStandardItem *
Snd_Controls_Model::item_plugs ( ) const
{
	return _item_plugs;
}


inline
int
Snd_Controls_Model::num_cards ( ) const
{
	return _card_infos.size();
}


inline
QModelIndex
Snd_Controls_Model::default_index ( ) const
{
	return _default_index;
}


inline
const QSnd::Snd_Control_Info &
Snd_Controls_Model::default_ctl ( ) const
{
	return _default_ctl;
}


inline
int
Snd_Controls_Model::num_controls ( ) const
{
	return _controls_plugs.size();
}


} // End of namespace


#endif


