//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_snd_mixer_simple_hpp__
#define __INC_snd_mixer_simple_hpp__

#include <QObject>
#include <QList>
#include <QVector>
#include <QSocketNotifier>

#include "snd_mixer_simple_elem.hpp"
#include "mixer_events.hpp"


namespace QSnd
{


///
/// @brief Snd_Mixer_Simple
///
/// Brings Qt and ALSA objects together but without
/// any GUI objects
///
class Snd_Mixer_Simple :
    public QObject
{
	Q_OBJECT

	// Public methods
	public:

	Snd_Mixer_Simple (
		QObject * parent_n = 0 );

	~Snd_Mixer_Simple ( );


	snd_hctl_t *
	snd_hctl ( );

	snd_mixer_t *
	snd_mixer ( );


	const QList < Snd_Mixer_Simple_Elem * > &
	elems ( );


	// Init / close

	void
	close_mixer ( );

	/// @return On an error a negative error code
	int
	open_mixer (
		const QString & dev_name_n );

	bool
	is_open ( ) const;


	// Signals
	signals:

	void
	sig_mixer_values_changed ( );

	void
	sig_mixer_elements_changed ( );


	// Public slots
	public slots:

	void
	element_changed ( );


	// Protected slots
	protected slots:

	void
	socket_event (
		int socket_id );


	// Protected methods
	protected:

	int
	create_qsnd_mixer_selems ( );

	/// @brief Filters elements from beeing used
	/// @return 1 if the element didn't pass the filter 0 otherwise
	int
	filter_mixer_elem (
		snd_mixer_elem_t * elem_n );

	int
	create_socket_notifiers ( );

	void
	set_socked_notifiers_enabled (
		bool flag_n );

	bool
	update_all_mixer_values ( );

	void
	print_snd_error (
		const QString & err_msg_n,
		int err_n );

	bool
	event (
		QEvent * event_n );


	// Private attributes
	private:

	snd_hctl_t * _snd_hctl;

	snd_mixer_t * _snd_mixer;

	QList < Snd_Mixer_Simple_Elem * > _mixer_simple_elems;

	QVector < pollfd > _pollfds;

	QList < QSocketNotifier * > _socket_notifiers;

	bool _update_requested;

	bool _signal_changes_requested;
};


inline
bool
Snd_Mixer_Simple::is_open ( ) const
{
	return ( _snd_mixer != 0 );
}


inline
snd_hctl_t *
Snd_Mixer_Simple::snd_hctl ( )
{
	return _snd_hctl;
}


inline
snd_mixer_t *
Snd_Mixer_Simple::snd_mixer ( )
{
	return _snd_mixer;
}


inline
const QList < Snd_Mixer_Simple_Elem * > &
Snd_Mixer_Simple::elems ( )
{
	return _mixer_simple_elems;
}


inline
void
Snd_Mixer_Simple::print_snd_error (
	const QString & err_msg_n,
	int err_n )
{
	print_alsa_error (
		err_msg_n.toLocal8Bit().constData(), err_n );
}


//
// Alsa callbacks
//

int
alsa_callback_mixer (
	snd_mixer_t * mixer,
	unsigned int mask,
	snd_mixer_elem_t * elem );


} // End of namespace


#endif


