//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "snd_mixer_simple_elem.hpp"

#include <iostream>

#include <QCoreApplication>
#include <QEvent>
#include "mixer_events.hpp"


namespace QSnd
{


//
// Snd_Mixer_Simple_Elem
//

Snd_Mixer_Simple_Elem::Snd_Mixer_Simple_Elem (
	QObject * parent_n,
	snd_mixer_t * mixer_n,
	snd_mixer_selem_id_t * elem_id_n ) :
QObject ( parent_n ),
_snd_mixer ( mixer_n )
{
	snd_mixer_selem_id_malloc ( &_snd_mixer_selem_id );
	set_snd_mixer_selem_id ( elem_id_n );
}


Snd_Mixer_Simple_Elem::~Snd_Mixer_Simple_Elem ( )
{
	snd_mixer_selem_id_free ( _snd_mixer_selem_id );
}


void
Snd_Mixer_Simple_Elem::clear ( )
{
	_elem_name.clear();

	for ( int ii=0; ii < 2; ++ii )
		_has_volume[ii] = false;
	for ( int ii=0; ii < 2; ++ii )
		_has_dB[ii] = false;
	for ( int ii=0; ii < 2; ++ii )
		_has_switch[ii] = false;
	for ( int ii=0; ii < 2; ++ii )
		_has_enum[ii] = false;


	for ( int ii=0; ii < 2; ++ii )
		_volume_joined[2] = false;
	for ( int ii=0; ii < 2; ++ii )
		_switch_joined[2] = false;
	for ( int ii=0; ii < 2; ++ii )
		_enum_joined[2] = false;

	_is_active = true;

	for ( int ii=0; ii < 2; ++ii )
		_volumes_equal[ii] = true;
	for ( int ii=0; ii < 2; ++ii )
		_switches_equal[ii] = true;
	for ( int ii=0; ii < 2; ++ii )
		_enums_equal[ii] = true;


	for ( int ii=0; ii < 2; ++ii )
		_volume_min[ii] = 0;
	for ( int ii=0; ii < 2; ++ii )
		_volume_max[ii] = 0;

	for ( int ii=0; ii < 2; ++ii )
		_dB_min[ii] = 0;
	for ( int ii=0; ii < 2; ++ii )
		_dB_max[ii] = 0;

	_enum_num_items = 0;
	_enum_item_names.clear();


	for ( int ii=0; ii < 2; ++ii )
		_channels[ii].clear();
	for ( int ii=0; ii < 2; ++ii )
		_volumes[ii].clear();
	for ( int ii=0; ii < 2; ++ii )
		_dB_values[ii].clear();
	for ( int ii=0; ii < 2; ++ii )
		_switch_states[ii].clear();
	for ( int ii=0; ii < 2; ++ii )
		_enum_values[ii].clear();
}


unsigned int
Snd_Mixer_Simple_Elem::elem_index ( ) const
{
	unsigned int res ( 0 );
	if ( snd_mixer_elem() != 0 ) {
		res = snd_mixer_selem_get_index ( snd_mixer_elem() );
	}
	return res;
}


const char *
Snd_Mixer_Simple_Elem::channel_name (
	unsigned int snd_dir_n,
	unsigned int channel_idx_n ) const
{
	const char * res ( 0 );
	if ( channel_idx_n < _channels[snd_dir_n].size() ) {
		res = snd_mixer_selem_channel_name ( _channels[snd_dir_n][channel_idx_n] );
	}
	return res;
}


void
Snd_Mixer_Simple_Elem::set_display_name (
	const QString & name_n )
{
	_display_name = name_n;
}


void
Snd_Mixer_Simple_Elem::set_snd_mixer_selem_id (
	snd_mixer_selem_id_t * selem_id_n )
{
	clear();

	if ( selem_id_n == 0 ) {
		return;
	}

	snd_mixer_selem_id_copy ( _snd_mixer_selem_id, selem_id_n );
	snd_mixer_elem_t * snd_elem = snd_mixer_elem();

	if ( snd_elem == 0 ) {
		return;
	}

	_elem_name = snd_mixer_selem_get_name ( snd_elem );
	_display_name = QCoreApplication::translate (
		"ALSA::Elem_Name", snd_mixer_selem_get_name ( snd_elem ) );

	for ( int ii=0; ii <= SND_MIXER_SCHN_LAST; ++ii ) {
		snd_mixer_selem_channel_id_t ch_id (
			static_cast < snd_mixer_selem_channel_id_t > ( ii ) );

		if ( snd_mixer_selem_has_playback_channel ( snd_elem, ch_id ) ) {
			_channels[0].push_back ( ch_id );
		}

		if ( snd_mixer_selem_has_capture_channel ( snd_elem, ch_id ) ) {
			_channels[1].push_back ( ch_id );
		}
	}

	if ( snd_mixer_selem_has_playback_volume ( snd_elem ) > 0 ) {
		const int snd_dir ( 0 );
		if ( _channels[snd_dir].size() > 0 ) {
			_has_volume[snd_dir] = true;

			if ( snd_mixer_selem_has_playback_volume_joined ( snd_elem ) ) {
				_volume_joined[snd_dir] = true;
				_volumes[snd_dir].resize ( 1, 0 );
			} else {
				_volumes[snd_dir].resize ( _channels[snd_dir].size(), 0 );
			}

			snd_mixer_selem_get_playback_volume_range (
				snd_elem, &_volume_min[snd_dir], &_volume_max[snd_dir] );

			int err = snd_mixer_selem_get_playback_dB_range (
				snd_elem, &_dB_min[snd_dir], &_dB_max[snd_dir] );
			if ( err < 0 ) {
				_dB_min[snd_dir] = 0;
				_dB_max[snd_dir] = 0;
			} else {
				_has_dB[snd_dir] = true;
				_dB_values[snd_dir].resize ( _volumes[snd_dir].size(), 0 );
			}

		} else {

			QString msg ( "[WW] %1 has a playback volume but no playback channels\n" );
			msg = msg.arg ( _elem_name );
			std::cerr << msg.toLocal8Bit().constData();

		}
	}

	if ( snd_mixer_selem_has_capture_volume ( snd_elem ) > 0 ) {
		const int snd_dir ( 1 );
		if ( _channels[snd_dir].size() > 0 ) {
			_has_volume[snd_dir] = true;

			if ( snd_mixer_selem_has_capture_volume_joined ( snd_elem ) ) {
				_volume_joined[snd_dir] = true;
				_volumes[snd_dir].resize ( 1, 0 );
			} else {
				_volumes[snd_dir].resize ( _channels[snd_dir].size(), 0 );
			}

			snd_mixer_selem_get_capture_volume_range (
				snd_elem, &_volume_min[snd_dir], &_volume_max[snd_dir] );

			int err = snd_mixer_selem_get_capture_dB_range (
				snd_elem, &_dB_min[snd_dir], &_dB_max[snd_dir] );
			if ( err < 0 ) {
				_dB_min[snd_dir] = 0;
				_dB_max[snd_dir] = 0;
			} else {
				_has_dB[snd_dir] = true;
				_dB_values[snd_dir].resize ( _volumes[snd_dir].size(), 0 );
			}

		} else {

			QString msg ( "[WW] %1 has a capture volume but no capture channels\n" );
			msg = msg.arg ( _elem_name );
			std::cerr << msg.toLocal8Bit().constData();

		}
	}

	if ( snd_mixer_selem_has_playback_switch ( snd_elem ) > 0 ) {
		const int snd_dir ( 0 );
		if ( _channels[snd_dir].size() > 0 ) {
			_has_switch[snd_dir] = true;

			if ( snd_mixer_selem_has_playback_switch_joined ( snd_elem ) ) {
				_switch_joined[snd_dir] = true;
				_switch_states[snd_dir].resize ( 1, 0 );
			} else {
				_switch_states[snd_dir].resize ( _channels[snd_dir].size(), 0 );
			}

		} else {

			QString msg ( "[WW] %1 has a playback switch but no playback channels\n" );
			msg = msg.arg ( _elem_name );
			std::cerr << msg.toLocal8Bit().constData();

		}
	}

	if ( snd_mixer_selem_has_capture_switch ( snd_elem ) > 0 ) {
		const int snd_dir ( 1 );
		if ( _channels[snd_dir].size() > 0 ) {
			_has_switch[snd_dir] = true;

			if ( snd_mixer_selem_has_capture_switch_joined ( snd_elem ) ) {
				_switch_joined[snd_dir] = true;
				_switch_states[snd_dir].resize ( 1, false );
			} else {
				_switch_states[snd_dir].resize ( _channels[snd_dir].size(), false );
			}

		} else {

			QString msg ( "[WW] %1 has a capture switch but no capture channels\n" );
			msg = msg.arg ( _elem_name );
			std::cerr << msg.toLocal8Bit().constData();

		}
	}

	// Enumerated
	if ( snd_mixer_selem_is_enumerated ( snd_elem ) ) {

		bool is_play ( snd_mixer_selem_is_enum_playback ( snd_elem ) > 0 );
		bool is_cap ( snd_mixer_selem_is_enum_capture ( snd_elem ) > 0 );

		if ( !( is_play || is_cap ) ) {
			// Workaround
			// Try to determine the direction from the name
			QString fstr ( _elem_name.toLower() );
			int fidx ( fstr.indexOf ( "mic" ) );
			if ( fidx < 0 ) {
				fidx = fstr.indexOf ( "adc " );
			}
			if ( fidx < 0 ) {
				fidx = fstr.indexOf ( "capture" );
			}
			if ( fidx < 0 ) {
				fidx = fstr.indexOf ( "input source" );
			}
			if ( fidx >= 0 ) {
				is_cap = true;
			}

			if ( !is_cap ) {
				if ( _channels[0].size() > 0 ) {
					is_play = true;
				} else if ( _channels[0].size() > 0 ) {
					is_cap = true;
				} else {
					is_play = true;
				}
			}
		}

		// Changing the channels for broken alsa implementations
		if ( is_play ) {
			_enum_joined[0] = true;
			_channels[0].resize ( 1, SND_MIXER_SCHN_MONO );
			_enum_values[0].resize ( _channels[0].size(), 0 );
		}
		if ( is_cap ) {
			_enum_joined[1] = true;
			_channels[1].resize ( 1, SND_MIXER_SCHN_MONO );
			_enum_values[1].resize ( _channels[1].size(), 0 );
		}

		_has_enum[0] = is_play;
		_has_enum[1] = is_cap;

		// Read enum item names
		const int num_items ( snd_mixer_selem_get_enum_items ( snd_elem ) );
		if ( num_items > 0 ) {
			_enum_num_items = num_items;
			_enum_item_names.resize ( _enum_num_items );

			const unsigned int buff_size ( 256 );
			char buff[buff_size];

			for ( int ii=0; ii < num_items; ++ii ) {
				int err = snd_mixer_selem_get_enum_item_name (
					snd_elem, ii, buff_size, buff );
				if ( err >= 0 ) {
					_enum_item_names[ii] = QCoreApplication::translate (
						"ALSA::Enum_Value", buff );
				} else {
					print_alsa_error ( "snd_mixer_selem_get_enum_item_name", err );
				}
			}
		}
	}

}


void
Snd_Mixer_Simple_Elem::set_volume (
	int snd_dir_n,
	int channel_idx_n,
	long volume_n )
{
	int err;
	snd_mixer_elem_t * snd_elem ( snd_mixer_elem() );

	//std::cout << "Snd_Mixer_Simple_Elem::set_volume " << volume_n << "\n";

	if ( snd_dir_n == 0 ) {
		err = snd_mixer_selem_set_playback_volume (
			snd_elem, _channels[0][channel_idx_n], volume_n );
		if ( err < 0 ) {
			print_alsa_error ( "snd_mixer_selem_set_playback_volume", err );
		}
	} else {
		err = snd_mixer_selem_set_capture_volume (
			snd_elem, _channels[1][channel_idx_n], volume_n );
		if ( err < 0 ) {
			print_alsa_error ( "snd_mixer_selem_set_capture_volume", err );
		}
	}

	value_was_set();
}


void
Snd_Mixer_Simple_Elem::set_volume_all (
	int snd_dir_n,
	long volume_n )
{
	int err;
	snd_mixer_elem_t * snd_elem ( snd_mixer_elem() );

	//std::cout << "Snd_Mixer_Simple_Elem::set_volume_all " << volume_n << "\n";

	if ( snd_dir_n == 0 ) {
		err = snd_mixer_selem_set_playback_volume_all ( snd_elem, volume_n );
		if ( err < 0 ) {
			print_alsa_error ( "snd_mixer_selem_set_playback_volume_all", err );
		}
	} else {
		err = snd_mixer_selem_set_capture_volume_all ( snd_elem, volume_n );
		if ( err < 0 ) {
			print_alsa_error ( "snd_mixer_selem_set_capture_volume_all", err );
		}
	}

	value_was_set();
}


void
Snd_Mixer_Simple_Elem::level_volumes (
	int snd_dir_n )
{
	const Volume_Buffer & vb ( _volumes[snd_dir_n] );
	const long num_volumes ( vb.size() );

	if ( volumes_equal ( snd_dir_n ) || ( num_volumes < 2 ) ) {
		return;
	}

	long vol_avrg_accu ( 0 );
	long vol_mod_accu ( 0 );

	for ( int ii=0; ii < num_volumes; ++ii ) {
		const long & val ( vb[ii] );

		long vol_mod;
		if ( ( val >= num_volumes ) ||
			( val <= (-num_volumes) ) )
		{
			const long vol_avrg ( val / num_volumes );
			vol_avrg_accu += vol_avrg;
			vol_mod = val;
			vol_mod -= vol_avrg * ( num_volumes - 1 ); // To prevent an overflow
			vol_mod -= vol_avrg;
		} else {
			vol_mod = val;
		}

		vol_mod_accu += vol_mod;

		if ( ( vol_mod_accu >= num_volumes ) ||
			( vol_mod_accu <= (-num_volumes) ) )
		{
			const long vol_avrg ( vol_mod_accu / num_volumes );
			vol_avrg_accu += vol_avrg;
			vol_mod_accu -= ( vol_avrg * num_volumes );
		}
	}

	set_volume_all ( snd_dir_n, vol_avrg_accu );
}


long
Snd_Mixer_Simple_Elem::ask_dB_vol_nearest (
	int snd_dir_n,
	long dB_value_n )
{
	long res ( 0 );

	const long vol_above ( ask_dB_vol ( snd_dir_n, dB_value_n, 1 ) );
	const long vol_below ( ask_dB_vol ( snd_dir_n, dB_value_n, -1 ) );
	long db_above ( ask_vol_dB ( snd_dir_n, vol_above ) );
	long db_below ( ask_vol_dB ( snd_dir_n, vol_below ) );

	db_above = abs ( db_above - dB_value_n );
	db_below = abs ( dB_value_n - db_below );

	if ( db_above > db_below ) {
		res = vol_below;
	} else {
		res = vol_above;
	}

	return res;
}


void
Snd_Mixer_Simple_Elem::set_dB (
	int snd_dir_n,
	int channel_idx_n,
	long dB_val_n,
	int dir_n )
{
	int err;
	snd_mixer_elem_t * snd_elem ( snd_mixer_elem() );

	//std::cout << "Snd_Mixer_Simple_Elem::set_dB " << dB_val_n << "\n";

	if ( snd_dir_n == 0 ) {
		err = snd_mixer_selem_set_playback_dB (
			snd_elem, _channels[0][channel_idx_n], dB_val_n, dir_n );
		if ( err < 0 ) {
			print_alsa_error ( "snd_mixer_selem_set_playback_dB", err );
		}
	} else {
		err = snd_mixer_selem_set_capture_dB (
			snd_elem, _channels[1][channel_idx_n], dB_val_n, dir_n );
		if ( err < 0 ) {
			print_alsa_error ( "snd_mixer_selem_set_capture_dB", err );
		}
	}

	value_was_set();
}


void
Snd_Mixer_Simple_Elem::set_dB_all (
	int snd_dir_n,
	long dB_val_n,
	int dir_n )
{
	int err;
	snd_mixer_elem_t * snd_elem ( snd_mixer_elem() );

	//std::cout << "Snd_Mixer_Simple_Elem::set_dB_all " << dB_val_n << "\n";

	if ( snd_dir_n == 0 ) {
		err = snd_mixer_selem_set_playback_dB_all ( snd_elem, dB_val_n, dir_n );
		if ( err < 0 ) {
			print_alsa_error ( "snd_mixer_selem_set_playback_dB_all", err );
		}
	} else {
		err = snd_mixer_selem_set_capture_dB_all ( snd_elem, dB_val_n, dir_n );
		if ( err < 0 ) {
			print_alsa_error ( "snd_mixer_selem_set_capture_dB_all", err );
		}
	}

	value_was_set();
}


void
Snd_Mixer_Simple_Elem::set_switch (
	int snd_dir_n,
	int channel_idx_n,
	bool state_n )
{
	int err;
	snd_mixer_elem_t * snd_elem ( snd_mixer_elem() );

	//std::cout << "Snd_Mixer_Simple_Elem::set_switch " << state_n << "\n";

	if ( snd_dir_n == 0 ) {
		err = snd_mixer_selem_set_playback_switch (
			snd_elem, _channels[0][channel_idx_n], state_n );
		if ( err < 0 ) {
			print_alsa_error ( "snd_mixer_selem_set_playback_switch", err );
		}
	} else {
		err = snd_mixer_selem_set_capture_switch (
			snd_elem, _channels[1][channel_idx_n], state_n );
		if ( err < 0 ) {
			print_alsa_error ( "snd_mixer_selem_set_capture_switch", err );
		}
	}

	value_was_set();
}


void
Snd_Mixer_Simple_Elem::set_switch_all (
	int snd_dir_n,
	bool state_n )
{
	int err;
	snd_mixer_elem_t * snd_elem ( snd_mixer_elem() );

	//std::cout << "Snd_Mixer_Simple_Elem::set_switch_all " << state_n << "\n";

	if ( snd_dir_n == 0 ) {
		err = snd_mixer_selem_set_playback_switch_all ( snd_elem, state_n );
		if ( err < 0 ) {
			print_alsa_error ( "snd_mixer_selem_set_playback_switch_all", err );
		}
	} else {
		err = snd_mixer_selem_set_capture_switch_all ( snd_elem, state_n );
		if ( err < 0 ) {
			print_alsa_error ( "snd_mixer_selem_set_playback_switch_all", err );
		}
	}

	value_was_set();
}


void
Snd_Mixer_Simple_Elem::level_switches (
	int snd_dir_n )
{
	const Switch_Buffer & swb ( _switch_states[snd_dir_n] );
	const unsigned int num_switches ( swb.size() );

	if ( switches_equal ( snd_dir_n ) || ( num_switches < 2 ) ) {
		return;
	}

	unsigned int num_on ( 0 );
	unsigned int num_off ( 0 );

	for ( unsigned int ii=0; ii < num_switches; ++ii ) {
		if ( swb[ii] ) {
			++num_on;
		} else {
			++num_off;
		}
	}

	set_switch_all ( snd_dir_n, ( num_on >= num_off ) );
}


void
Snd_Mixer_Simple_Elem::set_enum_index (
	int snd_dir_n,
	int channel_idx_n,
	Enum_Index index_n )
{
	int err;
	snd_mixer_elem_t * snd_elem ( snd_mixer_elem() );

	//std::cout << "Snd_Mixer_Simple_Elem::set_enum_index " << index_n << "\n";

	err = snd_mixer_selem_set_enum_item (
		snd_elem, _channels[snd_dir_n][channel_idx_n], index_n );
	if ( err < 0 ) {
		print_alsa_error ( "snd_mixer_selem_set_enum_item", err );
	}

	value_was_set();
}


void
Snd_Mixer_Simple_Elem::set_enum_index_all (
	int snd_dir_n,
	Enum_Index index_n )
{
	int err;
	snd_mixer_elem_t * snd_elem ( snd_mixer_elem() );

	//std::cout << "Snd_Mixer_Simple_Elem::set_enum_index_all " << index_n << "\n";

	for ( unsigned int ii=0; ii < _channels[snd_dir_n].size(); ++ii ) {
		err = snd_mixer_selem_set_enum_item (
			snd_elem, _channels[snd_dir_n][ii], index_n );
		if ( err < 0 ) {
			print_alsa_error ( "snd_mixer_selem_set_enum_item", err );
		}
	}

	value_was_set();
}


void
Snd_Mixer_Simple_Elem::value_was_set ( )
{
	if ( parent() != 0 ) {
		QEvent ev_req ( event_type_update_request );
		QCoreApplication::sendEvent ( parent(), &ev_req );
	}
}



bool
Snd_Mixer_Simple_Elem::update_mixer_values ( )
{
	bool changed ( false );

	snd_mixer_elem_t * snd_elem = snd_mixer_elem();
	if ( snd_elem == 0 ) {
		return changed;
	}

	//std::cout << "Snd_Mixer_Simple_Elem::update_mixer_values" << "\n";

	// Playback volume
	if ( _has_volume[0] ) {
		const Channel_Buffer & chs ( _channels[0] );
		Volume_Buffer & vols_pb ( _volumes[0] );
		for ( unsigned int ii=0; ii < vols_pb.size(); ++ii ) {
			long val ( 0 );
			int err ( snd_mixer_selem_get_playback_volume (
				snd_elem, chs[ii], &val ) );
			if ( err >= 0 ) {
				if ( vols_pb[ii] != val ) {
					vols_pb[ii] = val;
					changed = true;
				}
			} else {
				print_alsa_error ( "snd_mixer_selem_get_playback_volume", err );
			}
		}

		_volumes_equal[0] = true;
		for ( unsigned int ii=1; ii < vols_pb.size(); ++ii ) {
			if ( vols_pb[ii] != vols_pb[0] ) {
				_volumes_equal[0] = false;
				break;
			}
		}
	}

	// Capture volume
	if ( _has_volume[1] ) {
		const Channel_Buffer & chs ( _channels[1] );
		Volume_Buffer & vols_cap ( _volumes[1] );
		for ( unsigned int ii=0; ii < vols_cap.size(); ++ii ) {
			long val ( 0 );
			int err ( snd_mixer_selem_get_capture_volume (
				snd_elem, chs[ii], &val ) );
			if ( err >= 0 ) {
				if ( vols_cap[ii] != val ) {
					vols_cap[ii] = val;
					changed = true;
				}
			} else {
				print_alsa_error ( "snd_mixer_selem_get_capture_volume", err );
			}
		}

		_volumes_equal[1] = true;
		for ( unsigned int ii=1; ii < vols_cap.size(); ++ii ) {
			if ( vols_cap[ii] != vols_cap[0] ) {
				_volumes_equal[1] = false;
				break;
			}
		}
	}

	// Playback Decibel values
	if ( _has_dB[0] ) {
		const Channel_Buffer & chs ( _channels[0] );
		DB_Buffer & dBs_pb ( _dB_values[0] );
		for ( unsigned int ii=0; ii < dBs_pb.size(); ++ii ) {
			long val ( 0 );
			int err ( snd_mixer_selem_get_playback_dB (
				snd_elem, chs[ii], &val ) );
			if ( err >= 0 ) {
				if ( dBs_pb[ii] != val ) {
					dBs_pb[ii] = val;
					changed = true;
				}
			} else {
				_has_dB[0] = false;
			}
		}
	}

	// Capture Decibel values
	if ( _has_dB[1] ) {
		const Channel_Buffer & chs ( _channels[1] );
		DB_Buffer & dBs_cap ( _dB_values[1] );
		for ( unsigned int ii=0; ii < dBs_cap.size(); ++ii ) {
			long val ( 0 );
			int err ( snd_mixer_selem_get_capture_dB (
				snd_elem, chs[ii], &val ) );
			if ( err >= 0 ) {
				if ( dBs_cap[ii] != val ) {
					dBs_cap[ii] = val;
					changed = true;
				}
			} else {
				_has_dB[1] = false;
			}
		}
	}

	// Playback switch
	if ( _has_switch[0] ) {
		const Channel_Buffer & chs ( _channels[0] );
		Switch_Buffer & sws ( _switch_states[0] );
		for ( unsigned int ii=0; ii < sws.size(); ++ii ) {
			int val ( 0 );
			int err ( snd_mixer_selem_get_playback_switch (
				snd_elem, chs[ii], &val ) );
			if ( err >= 0 ) {
				const bool valb ( val != 0);
				if ( sws[ii] != valb ) {
					sws[ii] = valb;
					changed = true;
				}
			} else {
				print_alsa_error ( "snd_mixer_selem_get_playback_switch", err );
			};
		}

		_switches_equal[0] = true;
		for ( unsigned int ii=1; ii < sws.size(); ++ii ) {
			if ( sws[ii] != sws[0] ) {
				_switches_equal[0] = false;
				break;
			}
		}
	}

	// Capture switch
	if ( _has_switch[1] ) {
		const Channel_Buffer & chs ( _channels[1] );
		Switch_Buffer & sws ( _switch_states[1] );
		for ( unsigned int ii=0; ii < sws.size(); ++ii ) {
			int val ( 0 );
			int err ( snd_mixer_selem_get_capture_switch (
				snd_elem, chs[ii], &val ) );
			if ( err >= 0 ) {
				const bool valb ( val != 0);
				if ( sws[ii] != valb ) {
					sws[ii] = valb;
					changed = true;
				}
			} else {
				print_alsa_error ( "snd_mixer_selem_get_capture_switch", err );
			};
		}

		_switches_equal[1] = true;
		for ( unsigned int ii=1; ii < sws.size(); ++ii ) {
			if ( sws[ii] != sws[0] ) {
				_switches_equal[1] = false;
				break;
			}
		}
	}


	// Playback enum
	if ( _has_enum[0] ) {
		const Channel_Buffer & chs ( _channels[0] );
		Enum_Buffer & eni ( _enum_values[0] );
		for ( unsigned int ii=0; ii < eni.size(); ++ii ) {
			Enum_Index val ( 0 );
			int err ( snd_mixer_selem_get_enum_item (
				snd_elem, chs[ii], &val ) );
			if ( err >= 0 ) {
				if ( eni[ii] != val ) {
					eni[ii] = val;
					changed = true;
				}
			} else {
				print_alsa_error ( "snd_mixer_selem_get_enum_item ", err );
			};
		}

		_enums_equal[0] = true;
		for ( unsigned int ii=1; ii < eni.size(); ++ii ) {
			if ( eni[ii] != eni[0] ) {
				_enums_equal[0] = false;
				break;
			}
		}
	}


	// Capture enum
	if ( _has_enum[1] ) {
		const Channel_Buffer & chs ( _channels[1] );
		Enum_Buffer & eni ( _enum_values[1] );
		for ( unsigned int ii=0; ii < eni.size(); ++ii ) {
			Enum_Index val ( 0 );
			int err ( snd_mixer_selem_get_enum_item (
				snd_elem, chs[ii], &val ) );
			if ( err >= 0 ) {
				if ( eni[ii] != val ) {
					eni[ii] = val;
					changed = true;
				}
			} else {
				print_alsa_error ( "snd_mixer_selem_get_enum_item ", err );
			};
		}

		_enums_equal[1] = true;
		for ( unsigned int ii=1; ii < eni.size(); ++ii ) {
			if ( eni[ii] != eni[0] ) {
				_enums_equal[1] = false;
				break;
			}
		}
	}


	// Is active flag
	{
		bool flag ( snd_mixer_selem_is_active ( snd_elem ) > 0 );
		if ( _is_active != flag ) {
			_is_active = flag;
			changed = true;
		}
	}

	return changed;
}


void
Snd_Mixer_Simple_Elem::element_changed ( )
{
	emit sig_element_changed();
}


//
// Alsa callbacks
//

int
alsa_callback_mixer_elem (
	snd_mixer_elem_t * elem_n,
	unsigned int mask_n )
{
	int res ( 0 );

	//std::cout << "alsa_callback_mixer_elem " << mask_n << "\n";

	Snd_Mixer_Simple_Elem * qsme;
	{
		void * priv ( snd_mixer_elem_get_callback_private ( elem_n ) );
		qsme = reinterpret_cast < Snd_Mixer_Simple_Elem * > ( priv );
	}

	if ( qsme != 0 ) {

		switch ( mask_n ) {
			case SND_CTL_EVENT_MASK_ADD :
			case SND_CTL_EVENT_MASK_INFO :
			case SND_CTL_EVENT_MASK_REMOVE:
			case SND_CTL_EVENT_MASK_TLV :
				qsme->element_changed();
				break;

			case SND_CTL_EVENT_MASK_VALUE:
				qsme->update_mixer_values();
				break;

			default:
				std::cerr << "alsa_callback_mixer_elem: Unknown mask ( " << mask_n << " )\n";
				res = -1;
				break;
		}
	}

	return res;
}


} // End of namespace

