//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_switches_pad_proxy_group_hpp__
#define __INC_switches_pad_proxy_group_hpp__

#include <QObject>
#include <QString>
#include <QList>

#include "switches_pad_proxy.hpp"


namespace Wdg
{


///
/// @brief Switches_Pad_Proxies_Group
///
class Switches_Pad_Proxies_Group :
	public QObject
{
	Q_OBJECT


	// Public typedefs
	public:

	typedef QList < Switches_Pad_Proxy * > Proxy_List;


	// Public methods
	public:

	Switches_Pad_Proxies_Group (
		QObject * parent_n = 0 );

	virtual
	~Switches_Pad_Proxies_Group ( );


	// Proxies

	void
	clear_proxies ( );

	const Proxy_List &
	proxies ( ) const;

	void
	append_proxy (
		Switches_Pad_Proxy * proxy_n );

	Switches_Pad_Proxy *
	proxy (
		int idx_n );


	// Type flags

	bool
	is_switch ( ) const;

	void
	set_is_switch (
		bool flag_n );

	bool
	is_enum ( ) const;

	void
	set_is_enum (
		bool flag_n );


	bool
	is_multichannel ( );

	void
	set_is_multichannel (
		bool flag_n );


	// Group name

	const QString &
	group_name ( ) const;

	void
	set_group_name (
		const QString & name_n );


	// Tool tip

	const QString &
	tool_tip ( ) const;

	void
	set_tool_tip (
		const QString & tip_n );


	// Private attributes
	private:

	Proxy_List _proxies;

	QString _group_name;

	QString _tool_tip;

	bool _is_switch;

	bool _is_enum;
};


inline
const Switches_Pad_Proxies_Group::Proxy_List &
Switches_Pad_Proxies_Group::proxies ( ) const
{
	return _proxies;
}


inline
Switches_Pad_Proxy *
Switches_Pad_Proxies_Group::proxy (
	int idx_n )
{
	return _proxies[idx_n];
}


inline
const QString &
Switches_Pad_Proxies_Group::group_name ( ) const
{
	return _group_name;
}


inline
void
Switches_Pad_Proxies_Group::set_group_name (
	const QString & name_n )
{
	_group_name = name_n;
}


inline
const QString &
Switches_Pad_Proxies_Group::tool_tip ( ) const
{
	return _tool_tip;
}


inline
void
Switches_Pad_Proxies_Group::set_tool_tip (
	const QString & tip_n )
{
	_tool_tip = tip_n;
}


inline
bool
Switches_Pad_Proxies_Group::is_switch ( ) const
{
	return _is_switch;
}


inline
bool
Switches_Pad_Proxies_Group::is_enum ( ) const
{
	return _is_enum;
}


inline
bool
Switches_Pad_Proxies_Group::is_multichannel ( )
{
	return ( _proxies.size() > 1 );
}


typedef QList < Switches_Pad_Proxies_Group * > Switches_Pad_Proxy_Group_List;


} // End of namespace


#endif


