//
// C++ Implementation:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "switches_pad_widgets_group.hpp"

#include <QEvent>
#include <QFocusEvent>

#include <iostream>

namespace Wdg
{


//
// Switches_Pad_Widgets_Group
//
Switches_Pad_Widgets_Group::Switches_Pad_Widgets_Group (
	QWidget * parent_n ) :
QWidget ( parent_n ),
_label ( 0 ),
_lay_grid ( 0 ),
_focus_widgets ( 0 ),
_focus_widgets_idx ( 0 )
{
	setContextMenuPolicy ( Qt::NoContextMenu );

	QVBoxLayout * lay_vbox ( new QVBoxLayout );
	lay_vbox->setContentsMargins ( 0, 0, 0, 0 );
	setLayout ( lay_vbox );
}


Switches_Pad_Widgets_Group::~Switches_Pad_Widgets_Group ( )
{
	clear_widgets();
}


void
Switches_Pad_Widgets_Group::clear_widgets ( )
{
	if ( _widgets.size() > 0 ) {
		for ( int ii=0; ii < _widgets.size(); ++ ii ) {
			delete _widgets[ii];
		}
		_widgets.clear();
	}
	if ( _label != 0 ) {
		delete _label;
		_label = 0;
	}
	if ( _lay_grid != 0 ) {
		delete _lay_grid;
		_lay_grid = 0;
	}
}


void
Switches_Pad_Widgets_Group::set_label (
	QLabel * wdg_n )
{
	if ( _label != 0 ) {
		delete _label;
	}

	_label = wdg_n;

	if ( _label != 0 ) {
		if ( _lay_grid == 0 ) {
			_lay_grid = new QGridLayout;
			_lay_grid->setContentsMargins ( enum_spacing()*2, 0, 0, 0 );
			_lay_grid->setHorizontalSpacing ( enum_spacing() );
		}
		lay_vbox()->addWidget ( _label );
		lay_vbox()->addLayout ( _lay_grid );
	} else {
		if ( _lay_grid != 0 ) {
			delete _lay_grid;
			_lay_grid = 0;
		}
	}
}


void
Switches_Pad_Widgets_Group::append_widgets (
	Switches_Pad_Widgets * spw_n )
{
	if ( spw_n == 0 ) {
		return;
	}
	if ( spw_n->input_wdg() == 0 ) {
		return;
	}

	if ( _lay_grid != 0 ) {
		const unsigned int row ( _widgets.size() );
		if ( spw_n->label_wdg() != 0 ) {
			_lay_grid->addWidget ( spw_n->label_wdg(), row, 0 );
			_lay_grid->addWidget ( spw_n->input_wdg(), row, 1 );
		} else {
			_lay_grid->addWidget ( spw_n->input_wdg(), row, 0 );
		}
	} else {
		if ( spw_n->label_wdg() != 0 ) {
			QHBoxLayout * lay_hbox ( new QHBoxLayout );
			lay_hbox->setContentsMargins ( 0, 0, 0, 0 );
			lay_hbox->setSpacing ( enum_spacing() );

			lay_hbox->addWidget ( spw_n->label_wdg() );
			lay_hbox->addWidget ( spw_n->input_wdg() );
			lay_hbox->addStretch ( 1 );

			lay_vbox()->addLayout ( lay_hbox );
		} else {
			lay_vbox()->addWidget ( spw_n->input_wdg() );
		}
	}

	_widgets.append ( spw_n );
}


int
Switches_Pad_Widgets_Group::enum_spacing ( ) const
{
	return fontMetrics().averageCharWidth() * 4 / 3;
}


bool
Switches_Pad_Widgets_Group::eventFilter (
	QObject * watched,
	QEvent * event )
{
	bool res ( false );
	QFocusEvent * fev ( dynamic_cast < QFocusEvent * > ( event ) );
	if ( fev != 0 ) {

		_focus_widgets = 0;
		_focus_widgets_idx = 0;

		if ( fev->gotFocus () ) {

			if ( ( watched == _label ) && ( _label != 0 ) ) {
				if ( _widgets.size() > 0 ) {
					_widgets[0]->input_wdg()->setFocus();
					res = true;
				}
			} else {
				// Find focus widgets
				for ( int ii=0; ii < _widgets.size(); ++ii ) {
					Switches_Pad_Widgets * spw ( _widgets[ii] );
					if ( spw->input_wdg()->hasFocus() ) {
						_focus_widgets = spw;
						_focus_widgets_idx = ii;
						break;
					}
				}
			}
		}
	}

	return res;
}



} // End of namespace

