//
// C++ Interface:
//
// Description:
//
//
// Author: Sebastian Holtermann <sebholt@xwmw.org>, (C) 2010
//
// Copyright: See COPYING file that comes with this distribution
//
//


#ifndef __INC_switches_pad_widgets_group_hpp__
#define __INC_switches_pad_widgets_group_hpp__

#include <QWidget>
#include <QLabel>
#include <QList>

#include <QVBoxLayout>
#include <QGridLayout>

#include "switches_pad_widgets.hpp"
#include "switches_pad_proxies_group.hpp"


namespace Wdg
{


///
/// @brief Switches_Pad_Widgets_Group
///
class Switches_Pad_Widgets_Group :
	public QWidget
{
	// Public typedefs
	public:

	typedef QList < Switches_Pad_Widgets * > Widgets_List;


	// Public methods
	public:

	Switches_Pad_Widgets_Group (
		QWidget * parent_n = 0 );

	~Switches_Pad_Widgets_Group ( );


	void
	clear_widgets ( );


	// Label

	QLabel *
	label ( );

	void
	set_label (
		QLabel * wdg_n );


	// Widgets list

	const Widgets_List &
	widgets ( ) const;

	void
	append_widgets (
		Switches_Pad_Widgets * wdgs_n );


	const Switches_Pad_Widgets *
	widgets (
		int idx_n ) const;

	Switches_Pad_Widgets *
	widgets (
		int idx_n );


	// Mass adjusting

	Switches_Pad_Widgets *
	focus_widgets ( ) const;

	unsigned int
	focus_widgets_idx ( ) const;


	// Protected methods
	private:

	bool
	eventFilter (
		QObject * watched,
		QEvent * event );


	QVBoxLayout *
	lay_vbox ( );

	int
	enum_spacing ( ) const;


	// Private attributes
	private:

	Widgets_List _widgets;

	QLabel * _label;

	QGridLayout * _lay_grid;

	Switches_Pad_Widgets * _focus_widgets;

	unsigned int _focus_widgets_idx;
};


inline
QLabel *
Switches_Pad_Widgets_Group::label ( )
{
	return _label;
}


inline
const Switches_Pad_Widgets_Group::Widgets_List &
Switches_Pad_Widgets_Group::widgets ( ) const
{
	return _widgets;
}


inline
const Switches_Pad_Widgets *
Switches_Pad_Widgets_Group::widgets (
	int idx_n ) const
{
	return _widgets[idx_n];
}


inline
Switches_Pad_Widgets *
Switches_Pad_Widgets_Group::widgets (
	int idx_n )
{
	return _widgets[idx_n];
}


inline
Switches_Pad_Widgets *
Switches_Pad_Widgets_Group::focus_widgets ( ) const
{
	return _focus_widgets;
}


inline
unsigned int
Switches_Pad_Widgets_Group::focus_widgets_idx ( ) const
{
	return _focus_widgets_idx;
}


inline
QVBoxLayout *
Switches_Pad_Widgets_Group::lay_vbox ( )
{
	return static_cast < QVBoxLayout * > ( layout() );
}


typedef QList < Switches_Pad_Widgets_Group * > Switches_Pad_Widgets_Group_List;


} // End of namespace


#endif


